;; Checking (ice-9 and-let-star).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint ice-9 and-let-star)
  #:use-module (guile-lint common)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1))


;;-----------------------------------------------------------------------------
;; and-let*

;; not sure if empty body is supposed to be allowed, guess yes
(lint-arity! and-let* '(1 0 #t))

(define (form-and-let* where expr)
  (let* ((binding-list (second expr))
	 (where        (where-follow where binding-list)))
    (and (or (list? binding-list)
	     (lint-error where "`and-let*' clauses must be a (proper) list: ~s" binding-list))
	 (apply and?
		(iterate-where
		 map-in-order where binding-list
		 (lambda (where binding)
		   (or (and (list? binding)   ;; (SYMBOL EXPR)
			    (= 2 (length binding))
			    (symbol? (first binding)))
		       (and (list? binding)   ;; (EXPR)
			    (= 1 (length binding)))
		       (symbol? binding)      ;; SYMBOL
		       (lint-error where "`and-let*' clause invalid: ~s" binding))))))))

(define (lint-and-let* where now expr env)
  (if (form-and-let* where expr)
      (let more ((binding-list (second expr))
		 (env          env))
	(if (null? binding-list)
	    (lint-body-list where now (cddr expr) env)  ;; BODY
	    (let* ((binding (first binding-list))
		   (where   (where-follow where binding)))
	      (if (list>=? binding 2)
		  (begin
		    (lint-object where now (second binding) env)
		    (lint-subenv env
				 (lambda (subenv)
				   (env-bind-one subenv
						 (where-follow where binding)
						 (first binding)
						 (lint-value where
							     (second binding)
							     env))
				   (more (cdr binding-list) subenv))))
		  (begin
		    ;; (SYM) is a test of SYM, not a call, hence lint-list
		    (if (pair? binding)
			(lint-list where now binding env)
			(lint-object where now binding env))
		    (more (cdr binding-list) env))))))))

(lint-handler! and-let* lint-and-let*)
