;; Common setups for (gtk-1.2 gdk) or (gtk-2.0 gdk).

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define type-vector-or-f (type-or type-vector type-f))


(set-object-property! gdk-display-get-default 'pure #t)

;; C only
;;
(lint-types! gdk-color-intern 'pure (defs-type-find 'GdkColor)
	     (type-or type-string (defs-type-find 'GdkColor)))
(lint-types! gdk-font-intern 'pure (defs-type-find 'GdkFont)
	     (type-or type-string (defs-type-find 'GdkFont)))

;; various bits which are only declared as SCM
;;
(lint-types-override! gdk-gc-get-values 'pure type-list
		      (defs-type-find 'GdkGC))
(lint-types-override! gdk-rectangle-intersect
		      'pure (defs-type-or-f type-gtk-rect)
		      type-gtk-rect type-gtk-rect)
(lint-types-override! gdk-draw-text type-unspecified
		      (defs-type-find 'GdkDrawable)
		      (defs-type-find 'GdkFont)
		      (defs-type-find 'GdkGC)
		      type-gtk-int
		      type-gtk-int
		      type-string)
(lint-types-override! gdk-mbstowcs 'pure type-vector-or-f type-string)
(lint-types-override! gdk-text-width 'pure type-gtk-int
		      (defs-type-find 'GdkFont) type-string)
(lint-types-override! gdk-text-measure 'pure type-gtk-int
		      (defs-type-find 'GdkFont) type-string)
(lint-types-override! gdk-text-height 'pure type-gtk-int
		      (defs-type-find 'GdkFont) type-string)
(lint-types-override! gdk-text-extents
		      'pure (type-list-of type-exact-integer-nonneg)
		      (defs-type-find 'GdkFont) type-string)

(lint-types-override! gdk-window-get-position 'pure type-gdk-point
		      (defs-type-find 'GdkWindow))
(lint-types-override! gdk-window-get-root-origin 'pure type-gdk-point
		      (defs-type-find 'GdkWindow))
(lint-types-override! gdk-window-get-size 'pure type-gdk-point
		      (defs-type-find 'GdkWindow))
(lint-types-override! gdk-window-get-origin 'pure type-gdk-point
		      (defs-type-find 'GdkWindow))
(lint-types-override! gdk-window-get-deskrelative-origin 'pure type-gdk-point
		      (defs-type-find 'GdkWindow))

(lint-types-override! gdk-window-get-children
		      'pure (type-list-of (defs-type-find 'GdkWindow))
		      (defs-type-find 'GdkWindow))

(lint-types-override! gdk-colormap-alloc-colors type-list
		      (defs-type-find 'GdkColormap)
		      type-list-or-vector type-any type-any)
(lint-types-override! gdk-colormap-free-colors type-unspecified
		      (defs-type-find 'GdkColormap)
		      type-list-or-vector)

(lint-types-override! gdk-event-string 'pure type-string-or-f
		      (defs-type-find 'GdkEvent))
(lint-types-override! gdk-event-message 'pure type-vector
		      (defs-type-find 'GdkEvent))

(lint-types-override! gdk-query-depths
		      'pure (type-list-of type-exact-integer-nonneg))
(lint-types-override! gdk-query-visual-types
		      'pure (type-list-of (defs-type-find 'GdkVisualType)))
(lint-types-override! gdk-list-visuals
		      'pure (type-list-of (defs-type-find 'GdkVisual)))

;; not sure: gdk-selection-property-get-interp

(lint-types-override! gdk-property-change type-unspecified
		      (defs-type-find 'GdkWindow) type-gtk-atom
		      type-gtk-atom type-gtk-int (defs-type-find 'GdkPropMode)
		      (type-or type-list type-vector type-string))
(lint-types-override! gdk-text-property-to-text-list
		      'pure (type-or (type-list-of type-string) type-f)
		      type-gtk-atom type-gtk-int type-string)


(set-object-property! gdk-cursor-new             'pure #t)
(set-object-property! gdk-cursor-new-from-pixmap 'pure #t)

(set-object-property! gdk-gc-new             'pure #t)
(set-object-property! gdk-gc-new-with-values 'pure #t)
(set-object-property! gdk-gc-copy            'pure #t)
