;; Checking (ice-9 receive).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint ice-9 receive)
  #:use-module (guile-lint common)
  #:use-module (ice-9 receive)
  #:use-module (srfi srfi-1))


;;-----------------------------------------------------------------------------
;; receive

(lint-arity! receive '(3 0 #t))

(lint-types! receive type-any
	     type-unevaluated type-values type-any)

(define (form-receive where expr)
  (or (and (list? (second expr))
	   (every symbol? (second expr)))
      (lint-error (where-follow where (second expr))
		  "`receive' bindings must be a (proper) list of symbols: ~s" (second expr))))

(define (lint-receive where now expr env)
  (if (form-receive where expr)
      (begin
	(lint-formals-duplicates where 'receive (second expr))
	(lint-object where now (third expr) env)
	(lint-subenv env
		     (lambda (subenv)
		       ;; formals considered always used, in case only some
		       ;; received values are actually wanted
		       (env-bind subenv where (second expr) #t #t)
		       (lint-body where now (cdddr expr) subenv))))))
(lint-handler! receive lint-receive)
