;; Running lint on self.

;; Copyright 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; Usage: guile -s self.scm files...

(use-modules (ice-9 and-let-star)
	     (ice-9 getopt-long)
	     (srfi srfi-1)
	     (srfi srfi-13))


;; trim PART from the right of STR
(define-public (string-trim-right-str str part)
  (if (string-suffix? part str)
      (string-drop-right str (string-length part))
      str))

;; as per `resolve-interface' in boot-9.scm
;; an error can be thrown if the module exists but there's bad stuff in it
(define-public (module-exists? spec)
  (if (pair? (first spec))
      (set! spec (first spec)))
  (and-let* ((module (resolve-module spec)))
    (module-public-interface module)))



(define opts (getopt-long (program-arguments)
			  '((guile  (value #t))
			    (srcdir (value #t)))))

(define files (option-ref opts '() '()))

;; loaded bits, not runnable modules
(set! files (lset-difference string=? files
			     '("gdk-common.scm" "gtk-common.scm"
			       "slib-setups.scm")))

(define (file-good? filename)
  (cond ((string-contains filename "/")
	 (module-exists?
	  (map! string->symbol
		(string-split
		 (string-trim-right-str filename ".scm")
		 #\/))))
	((string-prefix? "srfi" filename)
	 (module-exists?
	  (cons 'srfi (map! string->symbol
			    (string-split
			     (string-trim-right-str filename ".scm")
			     #\/)))))

	((member filename '("weak-vector.scm"))
	 (module-exists?
	  (cons 'ice-9 (map! string->symbol
			     (string-split
			      (string-trim-right-str filename ".scm")
			      #\/)))))
	(else
	 #t)))

(set! files (filter (lambda (filename)
		      (or (file-good? filename)
			  (begin
			    (format #t "skip ~a\n" filename)
			    #f)))
		    files))		

(and-let* ((srcdir (option-ref opts 'srcdir #f)))
  (set! files (map (lambda (filename)
		     (string-append srcdir
				    (if (string-suffix? srcdir "/") "" "/")
				    filename))
		   files)))

(define guile (option-ref opts 'guile "guile"))

(define command (cons* guile "-s" "guile-lint" files))
(format #t "~a\n" (string-join command))
(apply execlp (first command) command)
