;; Checking slib via the --slib option.

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; load into guile-user the same as the slib specified "guile -lguile.init"
;; does
;;
(define-module (guile-user))

(define-module (guile-lint slib-option)
  #:use-module (guile-user))

(define-module (guile-user))

;; SCHEME_LIBRARY_PATH is used if set, look for an "slib" directory in the
;; guile %load-path (usually a symlink to somewhere else).
;;
;; The normal slib install is to have .init files in together with the .scm
;; files, but in the debian packaged slib (version 3a3-2 at least) there's
;; an "init" subdirectory for the .init files.
;;
(primitive-load
 (let ((slib-path (getenv "SCHEME_LIBRARY_PATH")))
   (if (not slib-path)
       (cond ((%search-load-path "slib/guile.init")
	      => (lambda (filename)
		   (set! slib-path (dirname filename))))
	     ((%search-load-path "slib/init/guile.init")
	      => (lambda (filename)
		   (set! slib-path (dirname (dirname filename)))))
	     (else
	      (display "\
guile-lint: cannot find slib guile.init file in %load-path
set SCHEME_LIBRARY_PATH if it's not under the guile tree\n")
	      (exit 1))))

   (let ((filename (string-append slib-path "/init/guile.init")))
     (if (not (file-exists? filename))
	 (set! filename (string-append slib-path "/guile.init")))
     filename)))

(export define)

(define-module (guile-lint slib-option)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1))

(load "slib-setups.scm")


;;-----------------------------------------------------------------------------
;; define

;; guile.init makes a new definition of `define', set it up the same as the
;; real one
(set! lint-body-define-alist (acons define lint-define-allowed
				    lint-body-define-alist))
(lint-handler! define lint-define-public)


;;-----------------------------------------------------------------------------

(define-public (lint-slib-option-setups)
  (set! guile-module (resolve-module '(guile-user)))
  (set! guile-env (env-from-module #f guile-module)))
