;; Checking (srfi srfi-11).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-11)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-11))


;;-----------------------------------------------------------------------------
;; let-values

(lint-arity! let-values '(2 0 #t))

;; used for let*-values too
(define (form-let-values where name expr)
  (let* ((binding-list (second expr))
	 (where        (where-follow where binding-list)))
    (and (or (list? binding-list)
	     (lint-error where "`~a' bindings must be a (proper) list: ~s"
			 name binding-list))
	 (apply and?
		(iterate-where
		 map-in-order where binding-list
		 (lambda (where binding)
		   (or (and (list? binding)
			    (= 2 (length binding))
			    (or (symbol? (first binding))
				(list-of-symbols-improper? (first binding))))
		       (lint-error where "`~a' binding must be symbols and value: ~s" name binding))))))))

(define (lint-let-values where now expr env)
  (if (form-let-values where 'let-values expr)
      (let ((binding-list (second expr)))
	(lint-bindings-duplicates where 'let-values binding-list)

	;; values
	(for-each-bindings where binding-list
			   (lambda (where sym val)
			     (lint-object where now val env)))
	(lint-subenv env
		     (lambda (subenv)
		       ;; bindings
		       (for-each-bindings where (second expr)
					  (lambda (where symlst val)
					    (env-bind subenv where symlst
						      #f #t)))
		       ;; body
		       (lint-body where now (cddr expr) subenv))))))

(lint-handler! let-values lint-let-values)


;;-----------------------------------------------------------------------------
;; let*-values

;; let*-values is a define-syntax, so comes out as the `common-sc-macro',
;; change to a unique object we can set a handler for
;;
(if (defined? '@)
    (set! (@ (srfi srfi-11) let*-values)
	  (procedure->memoizing-macro (macro-transformer let*-values)))
    (set! let*-values
	  (procedure->memoizing-macro (macro-transformer let*-values))))

(lint-arity! let*-values '(2 0 #t))

(define (lint-let*-values where now expr env)
  (if (form-let-values where 'let*-values expr)
      (let loop ((binding-list (second expr))
		 (env          env))
	(lint-subenv env
		     (lambda (subenv)
		       (if (null? binding-list)
			   (lint-body where now (cddr expr) subenv)
			   (let* ((binding (first binding-list))
				  (symlst  (first binding))
				  (val     (second binding))
				  (where   (where-follow where binding)))
			     (lint-bindings-duplicates where 'let*-values
						       (list binding))
			     (lint-object where now val env)
			     ;; no attempt to eval multiple-values
			     (env-bind subenv where symlst #f #t)
			     (loop (cdr binding-list) subenv))))))))

(lint-handler! let*-values lint-let*-values)
