;; Checking (srfi srfi-13).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-13)
  #:use-module (guile-lint common)
  #:use-module (guile-lint srfi srfi-14) ;; type-srfi14-charset
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-13))



(define-public type-srfi13-char-predicate
  (type-or type-character type-srfi14-charset type-predicate))


;;-----------------------------------------------------------------------------
;; srfi-13 types

(lint-types! string= type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string<> type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string< type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string> type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string<= type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string>= type-any
	     type-string type-string type-exact-integer-nonneg)

(lint-types! string-ci= type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-ci<> type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-ci< type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-ci> type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-ci<= type-any
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-ci>= type-any
	     type-string type-string type-exact-integer-nonneg)

(lint-types! string-append/shared 'pure type-string type-string)

(lint-types! string-concatenate                'pure type-string
	     (type-list-of type-string))
(lint-types! string-concatenate-reverse        'pure type-string
	     (type-list-of type-string))
(lint-types! string-concatenate/shared         'pure type-string
	     (type-list-of type-string))
(lint-types! string-concatenate-reverse/shared 'pure type-string
	     (type-list-of type-string))

(lint-types! string-contains  type-exact-integer-nonneg-or-f
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-contains-ci  type-exact-integer-nonneg-or-f
	     type-string type-string type-exact-integer-nonneg)

(if (not (eq? string-downcase core:string-downcase))
    ;; extended srfi-13 version in guile 1.6
    (lint-types! string-downcase  type-string
		 type-string
		 type-exact-integer-nonneg type-exact-integer-nonneg))
(if (not (eq? string-downcase! core:string-downcase!))
    ;; extended srfi-13 version in guile 1.6
    (lint-types! string-downcase! type-string
		 type-string
		 type-exact-integer-nonneg type-exact-integer-nonneg))

(lint-types! string-drop  type-string
	     type-string type-exact-integer-nonneg)
(lint-types! string-drop-right  type-string
	     type-string type-exact-integer-nonneg)

(lint-types! string-hash type-exact-integer-nonneg
	     type-string type-exact-integer-nonneg)
(lint-types! string-hash-ci type-exact-integer-nonneg
	     type-string type-exact-integer-nonneg)

(if (not (eq? string-index core:string-index))
    (lint-types! string-index  type-exact-integer-nonneg-or-f
		 type-string type-srfi13-char-predicate
		 type-exact-integer-nonneg type-exact-integer-nonneg))
(lint-types! string-index-right  type-exact-integer-nonneg-or-f
	     type-string type-srfi13-char-predicate
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-types! string-pad  type-string
	     type-string type-exact-integer-nonneg type-character
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-pad-right  type-string
	     type-string type-exact-integer-nonneg type-character
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-types! string-prefix? type-boolean
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-prefix-length type-exact-integer-nonneg
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-prefix-length-ci type-exact-integer-nonneg
	     type-string type-string type-exact-integer-nonneg)

(lint-types! string-replace  type-string
	     type-string type-string type-exact-integer-nonneg)

(lint-types! string-suffix?           type-boolean
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-suffix-ci?        type-boolean
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-suffix-length     type-exact-integer-nonneg
	     type-string type-string type-exact-integer-nonneg)
(lint-types! string-suffix-length-ci  type-exact-integer-nonneg
	     type-string type-string type-exact-integer-nonneg)

(lint-types! string-take  type-string
	     type-string type-exact-integer-nonneg)
(lint-types! string-take-right  type-string
	     type-string type-exact-integer-nonneg)

(lint-types! string-titlecase  type-string
	     type-string type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-titlecase! type-string
	     type-string type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-types! string-tokenize  (type-list-of type-string)
	     type-string type-srfi14-charset type-exact-integer-nonneg)

(lint-types! string-trim  type-string
	     type-string type-srfi13-char-predicate type-exact-integer-nonneg)
(lint-types! string-trim-both  type-string
	     type-string type-srfi13-char-predicate type-exact-integer-nonneg)
(lint-types! string-trim-right  type-string
	     type-string type-srfi13-char-predicate type-exact-integer-nonneg)

(if (not (eq? string-upcase core:string-upcase))
    ;; extended srfi-13 version in guile 1.6
    (lint-types! string-upcase  type-string
		 type-string
		 type-exact-integer-nonneg type-exact-integer-nonneg))
(if (not (eq? string-upcase! core:string-upcase!))
    ;; extended srfi-13 version in guile 1.6
    (lint-types! string-upcase! type-string
		 type-string
		 type-exact-integer-nonneg type-exact-integer-nonneg))

(lint-types! string-xcopy!  type-string
	     type-string type-exact-integer-nonneg
	     type-string type-exact-integer-nonneg)

(lint-types! xsubstring  type-string
	     type-string type-exact-integer-nonneg)


;;-----------------------------------------------------------------------------
;; string-any - char/pred str start end

;; only (2 0 #t) in scheme wrapper around C code
(lint-arity! string-any '(2 2 #f))

(lint-types! string-any  type-any
	     type-srfi13-char-predicate type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(define (lint-string-any where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(char))
  (lint-list where now expr env))

(lint-handler! string-any lint-string-any)


;;-----------------------------------------------------------------------------
;; string-compare - str str proclt proceq procgt s1 e1 s2 e2
;; string-compare-ci

(lint-types! string-compare type-any
	     type-string type-string
	     type-procedure type-procedure type-procedure
	     type-exact-integer-nonneg)
(lint-types! string-compare-ci type-any
	     type-string type-string
	     type-procedure type-procedure type-procedure
	     type-exact-integer-nonneg)

(define (lint-string-compare where now expr env)
  (lint-procexpr-arglist where now (fourth expr) env '(idx))
  (lint-procexpr-arglist where now (fifth  expr) env '(idx))
  (lint-procexpr-arglist where now (sixth  expr) env '(idx))
  (lint-list where now expr env))

(lint-handler! string-compare    lint-string-compare)
(lint-handler! string-compare-ci lint-string-compare)


;;-----------------------------------------------------------------------------
;; string-delete - str char/pred start end

(lint-types! string-delete  type-string
	     type-string type-srfi13-char-predicate
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(define (lint-string-delete where now expr env)
  (lint-procexpr-arglist where now (third expr) env '(char))
  (lint-list where now expr env))

(lint-handler! string-delete lint-string-delete)


;;-----------------------------------------------------------------------------
;; string-every

;; only (2 0 #t) in scheme wrapper around C code
(lint-arity! string-every '(2 2 #f))

(lint-types! string-every  type-any
	     type-srfi13-char-predicate type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-handler! string-every lint-string-any)


;;-----------------------------------------------------------------------------
;; string-filter

(lint-types! string-filter  type-string
	     type-string type-srfi13-char-predicate
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-handler! string-filter lint-string-delete)


;;-----------------------------------------------------------------------------
;; string-fold
;; string-fold-right

(lint-types! string-fold        type-any
	     type-procedure type-any type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-fold-right        type-any
	     type-procedure type-any type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(define (lint-string-fold where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(char prev))
  (lint-list where now expr env))

(lint-handler! string-fold       lint-string-fold)
(lint-handler! string-fold-right lint-string-fold)


;;-----------------------------------------------------------------------------
;; string-for-each
;; string-for-each-index

(lint-types! string-for-each        type-unspecified
	     type-procedure type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-for-each-index  type-unspecified
	     type-procedure type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-handler! string-for-each       lint-string-any)
(lint-handler! string-for-each-index lint-string-any)


;;-----------------------------------------------------------------------------
;; string-join

(lint-types! string-join   'pure type-string
	     (type-list-of type-string) type-string type-symbol)
(lint-defaults! string-join #f " " 'infix)


;;-----------------------------------------------------------------------------
;; string-map
;; string-map!

(lint-types! string-map   'pure type-string
	     type-procedure type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-map!  type-unspecified
	     type-procedure type-string
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-handler! string-map  lint-string-any)
(lint-handler! string-map! lint-string-any)


;;-----------------------------------------------------------------------------
;; string-skip
;; string-skip-right

(lint-types! string-skip  'pure type-exact-integer-nonneg
	     type-string type-srfi13-char-predicate
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! string-skip-right  'pure type-exact-integer-nonneg
	     type-string type-srfi13-char-predicate
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-handler! string-skip       lint-string-delete)
(lint-handler! string-skip-right lint-string-delete)


;;-----------------------------------------------------------------------------
;; string-tabulate

(lint-types! string-tabulate  type-string
	     type-procedure type-exact-integer-nonneg)

(define (lint-string-tabulate where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(idx))
  (lint-list where now expr env))
(lint-handler! string-tabulate  lint-string-tabulate)


;;-----------------------------------------------------------------------------
;; string-unfold - pstop fcproc gseedproc seed basestr finalproc
;; string-unfold-right

(lint-types! string-unfold    type-string
	     type-procedure type-procedure type-procedure
	     type-any type-string type-procedure)

(define (lint-string-unfold where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(seed))
  (lint-procexpr-arglist where now (third expr) env '(seed))
  (lint-procexpr-arglist where now (fourth expr) env '(seed))
  (if (list>=? expr 7)
      (lint-procexpr-arglist where now (seventh expr) env '(seed)))
  (lint-list where now expr env))

(lint-handler! string-unfold       lint-string-unfold)
(lint-handler! string-unfold-right lint-string-unfold)
