;; Checking (srfi srfi-14).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-14)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-14))


(define-public type-srfi14-charset
  (make-type "charset"
	     #:test char-set?))

;; this is an inum, but pretend it's a distinct type
(define-public type-srfi14-cursor
  (make-type "charset cursor"))


(lint-types! char-set?  'pure type-boolean type-any)
(lint-types! char-set=  'pure type-boolean type-srfi14-charset)
(lint-types! char-set<= 'pure type-boolean type-srfi14-charset)
(lint-types! char-set-hash 'pure type-exact-integer-nonneg
	     type-srfi14-charset type-exact-integer-nonneg)

;;; Iterating over character sets
(lint-types! char-set-cursor 'pure type-srfi14-cursor type-srfi14-charset)
(lint-types! char-set-ref    'pure type-character
	     type-srfi14-charset type-srfi14-cursor)
(lint-types! char-set-cursor-next 'pure type-srfi14-cursor
	     type-srfi14-charset type-srfi14-cursor)

(lint-types! end-of-char-set? 'pure type-boolean type-srfi14-cursor)

(lint-types! char-set-copy  'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set       'pure type-srfi14-charset type-character)
(lint-types! list->char-set 'pure type-srfi14-charset
	     (type-list-of type-character))
(lint-types! string->char-set 'pure type-srfi14-charset type-string)
(lint-types! ucs-range->char-set 'pure type-srfi14-charset
	     type-exact-integer-nonneg type-exact-integer-nonneg
	     type-any type-srfi14-charset)


(lint-types! char-set-size 'pure type-exact-integer-nonneg type-srfi14-charset)
(lint-types! char-set->list 'pure (type-list-of type-character)
	     type-srfi14-charset)
(lint-types! char-set->string 'pure type-string type-srfi14-charset)
(lint-types! char-set-contains? 'pure type-boolean
	     type-srfi14-charset type-character)

(lint-types! char-set-adjoin  'pure type-srfi14-charset
	     type-srfi14-charset type-character)
(lint-types! char-set-adjoin! type-srfi14-charset
	     type-srfi14-charset type-character)
(lint-types! char-set-delete  'pure type-srfi14-charset
	     type-srfi14-charset type-character)
(lint-types! char-set-delete! type-srfi14-charset
	     type-srfi14-charset type-character)
(lint-types! char-set-complement 'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-union      'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-intersection 'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-difference 'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-xor 'pure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-diff+intersection 'pure type-values type-srfi14-charset)
(lint-types! char-set-complement!   type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-union!        type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-intersection! type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-difference!   type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-xor!          type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-diff+intersection! type-values type-srfi14-charset)


;; for procedures like (char-set-for-each PRED CHARSET)
(define (lint-char-set-calling-proc-cs where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(char))
  (lint-list where now expr env))


;;-----------------------------------------------------------------------------
;; char-set-any

(lint-types! char-set-any type-any type-procedure type-srfi14-charset)
(lint-handler! char-set-any lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-count

(lint-types! char-set-count type-exact-integer-nonneg
	     type-procedure type-srfi14-charset)
(lint-handler! char-set-count lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-every

(lint-types! char-set-every type-any type-procedure type-srfi14-charset)
(lint-handler! char-set-every lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-filter
;; char-set-filter!

(lint-types! char-set-filter  type-srfi14-charset
	     type-procedure type-srfi14-charset type-srfi14-charset)
(lint-types! char-set-filter! type-srfi14-charset
	     type-procedure type-srfi14-charset type-srfi14-charset)
(lint-handler! char-set-filter  lint-char-set-calling-proc-cs)
(lint-handler! char-set-filter! lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-fold

(lint-types! char-set-fold type-any
	     type-procedure type-any type-srfi14-charset)

(define (lint-char-set-fold where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(char prev))
  (lint-list where now expr env))
(lint-handler! char-set-fold lint-char-set-fold)


;;-----------------------------------------------------------------------------
;; char-set-for-each

(lint-types! char-set-for-each type-unspecified
	     type-procedure type-srfi14-charset)
(lint-handler! char-set-for-each lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-map

(lint-types! char-set-map type-list
	     type-procedure type-srfi14-charset)
(lint-handler! char-set-map lint-char-set-calling-proc-cs)


;;-----------------------------------------------------------------------------
;; char-set-unfold
;; char-set-unfold!

(lint-types! char-set-unfold type-srfi14-charset
	     type-procedure type-procedure type-procedure
	     type-any type-srfi14-charset)

(define (lint-char-set-unfold where now expr env)
  (lint-procexpr-arglist where now (second expr) env '(seed))
  (lint-procexpr-arglist where now (third expr) env '(seed))
  (lint-procexpr-arglist where now (fourth expr) env '(seed))
  (lint-list where now expr env))

(lint-handler! char-set-unfold  lint-char-set-unfold)
(lint-handler! char-set-unfold! lint-char-set-unfold)
