;; Checking (srfi srfi-16).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-16)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-16))


;;-----------------------------------------------------------------------------
;; case-lambda

;; at least one clause
(lint-arity! case-lambda '(1 0 #t))

;; fprev and fnext are lambda style formals lists
;; return true if fprev shadows fnext, in the sense that any set of actual
;; parameters which would suit fnext will also suit fprev
(define (formals-shadow? fprev fnext)
  (cond ((pair? fprev)
	 (cond ((pair? fnext)
		;; (x ...) (y ...)
		;; both require one arg, skip and consider rest
		(formals-shadow? (cdr fprev) (cdr fnext)))
	       (else
		;; (x ...) y
		;; (x ...) ()
		;; prev doesn't match 0 args, so doesn't shadow
		#f)))
	((null? fprev)
	 ;; () ()      - prev and next both 0 args, so #t shadow
	 ;; () y       - prev doesn't match 1 arg, so #f doesn't shadow
	 ;; () (y ...) - prev doesn't match 1 arg, so #f doesn't shadow
	 (null? fnext))
	(else ;; symbol
	 ;; x y
	 ;; x ()
	 ;; x (y ...)
	 ;; prev takes any args, shadows anything next would take
	 #t)))

(define (lint-case-lambda-shadow case-where clause-where clause clause-list)
  (let ((prev-clause (first clause-list)))
    (if (not (eq? clause prev-clause))
	(let ((clause-formals (first clause))
	      (prev-formals   (first prev-clause)))
	  (if (formals-shadow? prev-formals clause-formals)
	      (begin
		(lint-error (where-follow clause-where clause-formals)
			    "`case-lambda' clause shadowed by previous: ~a"
			    clause-formals)
		(lint-error (where-follow (where-follow case-where prev-clause)
					  prev-formals)
			    " shadowing clause is here: ~a" prev-formals))
	      (lint-case-lambda-shadow case-where clause-where clause
				       (cdr clause-list)))))))

(define (lint-case-lambda where now expr env)
  (for-each (lambda (clause)
	      (if (form-lambda (where-follow where clause) 'case-lambda
			       (cons 'lambda clause))
		  (let ((formals (first clause)))
		    (lint-formals-duplicates where 'case-lambda formals)
		    (lint-case-lambda-shadow where (where-follow where clause)
					     clause (cdr expr))
		    (lint-subenv env
				 (lambda (subenv)
				   (env-bind subenv where formals #t
					     #t) ;; args used
				   (lint-body where #f (cdr clause)
					      subenv))))))
	    (cdr expr)))

(lint-handler! case-lambda lint-case-lambda)
