;; Checking (srfi srfi-26).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-26)
  #:use-module (guile-lint common)
  #:use-module (ice-9 and-let-star)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-26))


;;-----------------------------------------------------------------------------
;; cut

;; friendlier error message for no args like (cut)
(lint-arity! cut '(1 0 #t))

;; used by `cute' too
(define (lint-check-<...>-last where name expr)
  (let ((slots (cdr expr)))
    (and-let* ((dots (memq '<...> slots))
	       (     (not (eq? dots (last-pair slots)))))
      (lint-error where "`~a' <...> can only be last: ~s" name expr))))

(define (lint-cut where now expr env)
  (lint-check-<...>-last where 'cut expr)
  (lint-list where #f ;; args not evaluated until called
	     (remove (lambda (elem)
		       (memq elem '(<> <...>)))
		     (cdr expr)) env))

;; formals per slots
(define (lint-cut/cute-return name expr)
  (let ((slots (cdr expr)))
    (set-procedure-name!
     name (lambda-with-args (fold (lambda (elem formals)
				    ;; sym for each <>
				    (if (eq? '<> elem)
					(cons (gensym "slot") formals)
					formals))
				  ;; . rest for final <...>
				  (if (eq? '<...> (last slots))
				      (gensym "rest")
				      '())
				  slots)))))

(lint-handler! cut lint-cut)
(lint-return! cut (lambda (expr)
		    (lint-cut/cute-return '[cut] expr)))


;;-----------------------------------------------------------------------------
;; cute

;; friendlier error message for no args like (cut)
(lint-arity! cute '(1 0 #t))

(define (lint-cute where now expr env)
  (lint-check-<...>-last where 'cute expr)
  (lint-list where now ;; args evaluated now, just once
	     (remove (lambda (elem)
		       (memq elem '(<> <...>)))
		     (cdr expr)) env))

(lint-handler! cute lint-cute)
(lint-return! cute (lambda (expr)
		     (lint-cut/cute-return '[cute] expr)))
