;; Checking (srfi srfi-34).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-34)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1)
  #:use-module ((srfi srfi-34)
		;; avoid clash with core `raise
		#:renamer (symbol-prefix-proc 'srfi34:)))


;;-----------------------------------------------------------------------------
;; guard

;; srfi-34.scm is only (1 0 #t), but body cannot be empty
(lint-arity! srfi34:guard '(2 0 #t))

(define (form-guard where expr)
  (let* ((var+clauses (second expr))
	 (where       (where-follow where var+clauses)))
    (and (or (list>=? var+clauses 2)
	     (lint-error where "`raise' takes var and 1 or more clauses: ~s"
			 var+clauses))
	 (or (symbol? (first var+clauses))
	     (lint-error (where-follow where (first var+clauses))
			 "`raise' variable must be a symbol: ~s"
			 (first var+clauses)))
	 (begin
	   (iterate-where
	    for-each where
	    (cdr var+clauses)
	    (lambda (where clause)
	      (or (and (pair? clause)  ;; non-empty proper list
		       (list? clause))
		  (lint-error where
			      "`raise' clause must be condition and forms: ~s"
			      clause))))))))

(define (lint-guard where now expr env)
  (if (form-guard where expr)
      (let ((var+clauses (second expr)))
	(lint-subenv env
		     (lambda (subenv)
		       (let ((where (where-follow where var+clauses))
			     (end   (last var+clauses)))
			 (env-bind-one env where (first var+clauses) #f #t)
			 (for-each (lambda (clause)
				     ;; last condition can be `else', change
				     ;; it to #t for the benefit of
				     ;; lint-body-list
				     (if (and (eq? end clause)
					      (pair? clause)
					      (eq? 'else (first clause)))
					 (set! clause (cons #t (cdr clause))))
				     (lint-body-list where now clause subenv
						     #:initial-used 1))
				   (cdr var+clauses)))))
	(lint-body where now (cddr expr) env))))

(lint-handler! srfi34:guard lint-guard)


;;-----------------------------------------------------------------------------
;; raise

(lint-types! srfi34:raise type-unspecified type-any)


;;-----------------------------------------------------------------------------
;; with-exception-handler

(define (lint-with-exception-handler where now expr env)
  (lint-procexpr-arglist where #f (second expr) env '(errobj))
  (lint-procexpr-arglist where #f (third expr) env '())
  (lint-list where now expr env))

(lint-handler! srfi34:with-exception-handler lint-with-exception-handler)
