;; Checking (srfi srfi-39).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-39)
  #:use-module (guile-lint common)
  #:use-module (ice-9 optargs)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-39))


;;-----------------------------------------------------------------------------
;; make-parameter

;; defined with case-lambda, and such arity is not checked (yet)
(lint-arity! make-parameter '(1 1 #f))

(lint-types! make-parameter 'pure type-procedure
	     type-any type-procedure)

(define (lint-make-parameter where now expr env)
  (if (= 3 (length expr))
      ;; called (CONV VALUE)
      (lint-procexpr-arglist where #f (third expr) env '(value)))
  (lint-list where now expr env))

(define* ([parameter] #:optional value) #f)
(lint-handler! make-parameter lint-make-parameter)
(lint-return! make-parameter (lambda (expr) [parameter]))


;;-----------------------------------------------------------------------------
;; parameterize

;; disambiguate syncase
(if (defined? '@)
    (set! (@ (srfi srfi-39) parameterize)
	  (procedure->memoizing-macro (macro-transformer parameterize)))
    (set! parameterize
	  (procedure->memoizing-macro (macro-transformer parameterize))))

;; macro
(lint-arity! parameterize '(2 0 #t))

(define (form-parameterize where expr)
  (let ((binding-list (second expr)))
    (set! where (where-follow where binding-list))
    (and (or (list? (second expr))
	     (lint-error where "`parameterize' bindings must be (proper) list: ~s" binding-list))

	 (apply and?
		(iterate-where
		 map-in-order where binding-list
		 (lambda (where elem)
		   (or (and (list? elem)
			    (= 2 (length elem)))
		       (lint-error where "`parameterize' binding must be two elements (PARAM VALUE): ~s" elem))))))))

(define (lint-parameterize where now expr env)
  (if (form-parameterize where expr)
      (iterate-where for-each where (second expr)
		     (lambda (where elem)
		       (lint-object where now elem env))))
  (lint-body where now (cddr expr) env))

(lint-handler! parameterize lint-parameterize)


;;-----------------------------------------------------------------------------
;; with-parameters*

(define (lint-with-parameters* where now expr env)
  (lint-procexpr-arglist where now (fourth expr) env '())
  (lint-list where now expr env))

(lint-handler! with-parameters* lint-with-parameters*)
