;; Checking (srfi srfi-4).

;; Copyright 2007, 2008 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-4)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-4))


(define-public type-uniform-vector
  (make-type "uniform vector"
	     #:test uniform-vector?))

;;-----------------------------------------------------------------------------
;; u8

(define-public type-u8vector
  (make-type "u8vector"
	     #:supertype type-uniform-vector
	     #:test u8vector?))

(define type-u8element type-exact-integer-nonneg)

(lint-types! make-u8vector   type-u8vector
	     type-exact-integer-nonneg type-u8element)
(lint-types! u8vector        type-u8vector type-u8element)
(lint-types! u8vector?       type-boolean type-any)
(lint-types! u8vector-length type-exact-integer-nonneg type-u8vector)
(lint-types! u8vector-ref    'pure type-u8element
	     type-u8vector type-exact-integer-nonneg)
(lint-types! u8vector-set!   type-unspecified
	     type-u8vector type-exact-integer-nonneg type-u8element)
(lint-types! u8vector->list 'pure (type-list-of type-u8element) type-u8vector)
(lint-types! list->u8vector 'pure type-u8vector (type-list-of type-u8element))
(lint-types! any->u8vector  'pure type-u8vector ;; new in guile 1.8
	     (type-or (type-list-of type-u8element)
		      (type-vector-of type-u8element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; s8

(define-public type-s8vector
  (make-type "s8vector"
	     #:supertype type-uniform-vector
	     #:test s8vector?))

(define type-s8element type-exact-integer)

(lint-types! make-s8vector   type-s8vector
	     type-exact-integer-nonneg type-s8element)
(lint-types! s8vector        type-s8vector type-s8element)
(lint-types! s8vector?       type-boolean type-any)
(lint-types! s8vector-length type-exact-integer-nonneg type-s8vector)
(lint-types! s8vector-ref    'pure type-s8element
	     type-s8vector type-exact-integer-nonneg)
(lint-types! s8vector-set!   type-unspecified
	     type-s8vector type-exact-integer-nonneg type-s8element)
(lint-types! s8vector->list 'pure (type-list-of type-s8element) type-s8vector)
(lint-types! list->s8vector 'pure type-s8vector (type-list-of type-s8element))
(lint-types! any->s8vector 'pure type-s8vector ;; new in guile 1.8
	     (type-or (type-list-of type-s8element)
		      (type-vector-of type-s8element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; u16

(define-public type-u16vector
  (make-type "u16vector"
	     #:supertype type-uniform-vector
	     #:test u16vector?))

(define type-u16element type-exact-integer-nonneg)

(lint-types! make-u16vector   type-u16vector
	     type-exact-integer-nonneg type-u16element)
(lint-types! u16vector        type-u16vector type-u16element)
(lint-types! u16vector?       type-boolean type-any)
(lint-types! u16vector-length type-exact-integer-nonneg type-u16vector)
(lint-types! u16vector-ref    'pure type-u16element
	     type-u16vector type-exact-integer-nonneg)
(lint-types! u16vector-set!   type-unspecified
	     type-u16vector type-exact-integer-nonneg type-u16element)
(lint-types! u16vector->list 'pure (type-list-of type-u16element) type-u16vector)
(lint-types! list->u16vector 'pure type-u16vector (type-list-of type-u16element))
(lint-types! any->u16vector  'pure type-u16vector ;; new in guile 1.8
	     (type-or (type-list-of type-u16element)
		      (type-vector-of type-u16element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; s16

(define-public type-s16vector
  (make-type "s16vector"
	     #:supertype type-uniform-vector
	     #:test s16vector?))

(define type-s16element type-exact-integer)

(lint-types! make-s16vector   type-s16vector
	     type-exact-integer-nonneg type-s16element)
(lint-types! s16vector        type-s16vector type-s16element)
(lint-types! s16vector?       type-boolean type-any)
(lint-types! s16vector-length type-exact-integer-nonneg type-s16vector)
(lint-types! s16vector-ref    'pure type-s16element
	     type-s16vector type-exact-integer-nonneg)
(lint-types! s16vector-set!   type-unspecified
	     type-s16vector type-exact-integer-nonneg type-s16element)
(lint-types! s16vector->list 'pure (type-list-of type-s16element) type-s16vector)
(lint-types! list->s16vector 'pure type-s16vector (type-list-of type-s16element))
(lint-types! any->s16vector  'pure type-s16vector ;; new in guile 1.8
	     (type-or (type-list-of type-s16element)
		      (type-vector-of type-s16element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; u32

(define-public type-u32vector
  (make-type "u32vector"
	     #:supertype type-uniform-vector
	     #:test u32vector?))

(define type-u32element type-exact-integer-nonneg)

(lint-types! make-u32vector   type-u32vector
	     type-exact-integer-nonneg type-u32element)
(lint-types! u32vector        type-u32vector type-u32element)
(lint-types! u32vector?       type-boolean type-any)
(lint-types! u32vector-length type-exact-integer-nonneg type-u32vector)
(lint-types! u32vector-ref    'pure type-u32element
	     type-u32vector type-exact-integer-nonneg)
(lint-types! u32vector-set!   type-unspecified
	     type-u32vector type-exact-integer-nonneg type-u32element)
(lint-types! u32vector->list 'pure (type-list-of type-u32element) type-u32vector)
(lint-types! list->u32vector 'pure type-u32vector (type-list-of type-u32element))
(lint-types! any->u32vector  'pure type-u32vector ;; new in guile 1.8
	     (type-or (type-list-of type-u32element)
		      (type-vector-of type-u32element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; s32

(define-public type-s32vector
  (make-type "s32vector"
	     #:supertype type-uniform-vector
	     #:test s32vector?))

(define type-s32element type-exact-integer)

(lint-types! make-s32vector   type-s32vector
	     type-exact-integer-nonneg type-s32element)
(lint-types! s32vector        type-s32vector type-s32element)
(lint-types! s32vector?       type-boolean type-any)
(lint-types! s32vector-length type-exact-integer-nonneg type-s32vector)
(lint-types! s32vector-ref    'pure type-s32element
	     type-s32vector type-exact-integer-nonneg)
(lint-types! s32vector-set!   type-unspecified
	     type-s32vector type-exact-integer-nonneg type-s32element)
(lint-types! s32vector->list 'pure (type-list-of type-s32element) type-s32vector)
(lint-types! list->s32vector 'pure type-s32vector (type-list-of type-s32element))
(lint-types! any->s32vector  'pure type-s32vector ;; new in guile 1.8
	     (type-or (type-list-of type-s32element)
		      (type-vector-of type-s32element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; u64

(define-public type-u64vector
  (make-type "u64vector"
	     #:supertype type-uniform-vector
	     #:test u64vector?))

(define type-u64element type-exact-integer-nonneg)

(lint-types! make-u64vector   type-u64vector
	     type-exact-integer-nonneg type-u64element)
(lint-types! u64vector        type-u64vector type-u64element)
(lint-types! u64vector?       type-boolean type-any)
(lint-types! u64vector-length type-exact-integer-nonneg type-u64vector)
(lint-types! u64vector-ref    'pure type-u64element
	     type-u64vector type-exact-integer-nonneg)
(lint-types! u64vector-set!   type-unspecified
	     type-u64vector type-exact-integer-nonneg type-u64element)
(lint-types! u64vector->list 'pure (type-list-of type-u64element) type-u64vector)
(lint-types! list->u64vector 'pure type-u64vector (type-list-of type-u64element))
(lint-types! any->u64vector  'pure type-u64vector ;; new in guile 1.8
	     (type-or (type-list-of type-u64element)
		      (type-vector-of type-u64element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; s64

(define-public type-s64vector
  (make-type "s64vector"
	     #:supertype type-uniform-vector
	     #:test s64vector?))

(define type-s64element type-exact-integer)

(lint-types! make-s64vector   type-s64vector
	     type-exact-integer-nonneg type-s64element)
(lint-types! s64vector        type-s64vector type-s64element)
(lint-types! s64vector?       type-boolean type-any)
(lint-types! s64vector-length type-exact-integer-nonneg type-s64vector)
(lint-types! s64vector-ref    'pure type-s64element
	     type-s64vector type-exact-integer-nonneg)
(lint-types! s64vector-set!   type-unspecified
	     type-s64vector type-exact-integer-nonneg type-s64element)
(lint-types! s64vector->list 'pure (type-list-of type-s64element) type-s64vector)
(lint-types! list->s64vector 'pure type-s64vector (type-list-of type-s64element))
(lint-types! any->s64vector  'pure type-s64vector ;; new in guile 1.8
	     (type-or (type-list-of type-s64element)
		      (type-vector-of type-s64element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; f32

(define-public type-f32vector
  (make-type "f32vector"
	     #:supertype type-uniform-vector
	     #:test f32vector?))

(define type-f32element type-real)

(lint-types! make-f32vector   type-f32vector
	     type-exact-integer-nonneg type-f32element)
(lint-types! f32vector        type-f32vector type-f32element)
(lint-types! f32vector?       type-boolean type-any)
(lint-types! f32vector-length type-exact-integer-nonneg type-f32vector)
(lint-types! f32vector-ref    'pure type-f32element
	     type-f32vector type-exact-integer-nonneg)
(lint-types! f32vector-set!   type-unspecified
	     type-f32vector type-exact-integer-nonneg type-f32element)
(lint-types! f32vector->list 'pure (type-list-of type-f32element) type-f32vector)
(lint-types! list->f32vector 'pure type-f32vector (type-list-of type-f32element))
(lint-types! any->f32vector  'pure type-f32vector ;; new in guile 1.8
	     (type-or (type-list-of type-f32element)
		      (type-vector-of type-f32element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; f64

(define-public type-f64vector
  (make-type "f64vector"
	     #:supertype type-uniform-vector
	     #:test f64vector?))

(define type-f64element type-real)

(lint-types! make-f64vector   type-f64vector
	     type-exact-integer-nonneg type-f64element)
(lint-types! f64vector        type-f64vector type-f64element)
(lint-types! f64vector?       type-boolean type-any)
(lint-types! f64vector-length type-exact-integer-nonneg type-f64vector)
(lint-types! f64vector-ref    'pure type-f64element
	     type-f64vector type-exact-integer-nonneg)
(lint-types! f64vector-set!   type-unspecified
	     type-f64vector type-exact-integer-nonneg type-f64element)
(lint-types! f64vector->list 'pure (type-list-of type-f64element) type-f64vector)
(lint-types! list->f64vector 'pure type-f64vector (type-list-of type-f64element))
(lint-types! any->f64vector  'pure type-f64vector ;; new in guile 1.8
	     (type-or (type-list-of type-f64element)
		      (type-vector-of type-f64element)
		      type-uniform-vector))


;;-----------------------------------------------------------------------------
;; c32

(if (defined? 'c32vector) ;; new in guile 1.8
    (begin
      (define-public type-c32vector
	(make-type "c32vector"
		   #:supertype type-uniform-vector
		   #:test c32vector?))

      (define type-c32element type-number)

      (lint-types! make-c32vector   type-c32vector
		   type-exact-integer-nonneg type-c32element)
      (lint-types! c32vector        type-c32vector type-c32element)
      (lint-types! c32vector?       type-boolean type-any)
      (lint-types! c32vector-length type-exact-integer-nonneg type-c32vector)
      (lint-types! c32vector-ref    'pure type-c32element
		   type-c32vector type-exact-integer-nonneg)
      (lint-types! c32vector-set!   type-unspecified
		   type-c32vector type-exact-integer-nonneg type-c32element)
      (lint-types! c32vector->list 'pure (type-list-of type-c32element)
		   type-c32vector)
      (lint-types! list->c32vector 'pure type-c32vector
		   (type-list-of type-c32element))
      (lint-types! any->c32vector  'pure type-c32vector
		   (type-or (type-list-of type-c32element)
			    (type-vector-of type-c32element)
			    type-uniform-vector))))


;;-----------------------------------------------------------------------------
;; c64

(if (defined? 'c64vector) ;; new in guile 1.8
    (begin
      (define-public type-c64vector
	(make-type "c64vector"
		   #:supertype type-uniform-vector
		   #:test c64vector?))

      (define type-c64element type-number)

      (lint-types! make-c64vector   type-c64vector
		   type-exact-integer-nonneg type-c64element)
      (lint-types! c64vector        type-c64vector type-c64element)
      (lint-types! c64vector?       type-boolean type-any)
      (lint-types! c64vector-length type-exact-integer-nonneg type-c64vector)
      (lint-types! c64vector-ref    'pure type-c64element
		   type-c64vector type-exact-integer-nonneg)
      (lint-types! c64vector-set!   type-unspecified
		   type-c64vector type-exact-integer-nonneg type-c64element)
      (lint-types! c64vector->list 'pure (type-list-of type-c64element)
		   type-c64vector)
      (lint-types! list->c64vector 'pure type-c64vector
		   (type-list-of type-c64element))
      (lint-types! any->c64vector  'pure type-c64vector
		   (type-or (type-list-of type-c64element)
			    (type-vector-of type-c64element)
			    type-uniform-vector))))
