;; Checking (srfi srfi-60).

;; Copyright 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-60)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-1)
  #:use-module ((srfi srfi-60)
		;; avoid warnings about override core bit-count
		#:renamer (symbol-prefix-proc 'srfi60:)))


;; Aliases for core funcs, already have type info on these.
;; bitwise-and
;; bitwise-ior
;; bitwise-xor
;; bitwise-not
;; any-bits-set?
;; bit-count
;; bit-set?
;; bit-field
;; arithmetic-shift

(lint-types! srfi60:bitwise-if 'pure type-exact-integer
	     type-exact-integer type-exact-integer)
;; bitwise-merge is an alias for bitwise-if

(lint-types! srfi60:log2-binary-factors 'pure type-exact-integer
	     type-exact-integer)
;; first-set-bit is an alias for log2-binary-factors

(lint-types! srfi60:copy-bit 'pure type-exact-integer 
	     type-exact-integer-nonneg type-exact-integer type-boolean)
(lint-types! srfi60:copy-bit-field 'pure type-exact-integer 
	     type-exact-integer type-exact-integer
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-types! srfi60:rotate-bit-field 'pure type-exact-integer 
	     type-exact-integer type-exact-integer
	     type-exact-integer-nonneg type-exact-integer-nonneg)
(lint-types! srfi60:reverse-bit-field 'pure type-exact-integer 
	     type-exact-integer
	     type-exact-integer-nonneg type-exact-integer-nonneg)

(lint-types! srfi60:integer->list 'pure type-list
	     type-exact-integer type-exact-integer-nonneg)

(lint-types! srfi60:list->integer 'pure type-exact-integer type-list)


;;-----------------------------------------------------------------------------
;; booleans->integer

(lint-types! srfi60:booleans->integer 'pure type-exact-integer type-boolean)

(let ((old-lint-apply lint-apply))
  (define (srfi-60-lint-apply where now expr env)
    (if (and (= 3 (length expr))
	     (eq? srfi60:booleans->integer
		  (lint-value where (second expr) env)))
	(lint-warn where "hint, `(apply booleans->integer LST)' can be `(list->integer LST)"))

    (old-lint-apply where now expr env))

  (set! lint-apply srfi-60-lint-apply)
  (set! (lint-handler-property apply) lint-apply))
