;; Checking (srfi srfi-9).

;; Copyright 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint srfi srfi-9)
  #:use-module (guile-lint common)
  #:use-module (srfi srfi-9))


;; as per Guile 1.8, made available in 1.6
(if (not (defined? '@@))
    (define-macro (@@ mod-name var-name)
      (let ((var (module-variable (resolve-module mod-name) var-name)))
	(if (not var)
	    (error "no such variable" (list '@@ mod-name var-name)))
	(list variable-ref var))))


(if (module-defined? (resolve-module '(srfi srfi-9))
		     'record-accessor-strict)  ;; guile 1.6.9
    (let ()
      (define ([record-accessor] record) #f)
      (lint-types! [record-accessor]  type-any type-record)

      (lint-types! (@@ (srfi srfi-9) record-accessor-strict)
		   'pure type-procedure
		   type-rectype type-symbol)
      (lint-return! (@@ (srfi srfi-9) record-accessor-strict)
		    (lambda (expr)
		      [record-accessor]))))

(if (module-defined? (resolve-module '(srfi srfi-9))
		     'record-modifier-strict)  ;; guile 1.6.9
    (let ()
      (define ([record-modifier] record value) #f)
      (lint-types! [record-modifier]  type-unspecified type-record type-any)

      (lint-types! (@@ (srfi srfi-9) record-modifier-strict)
		   'pure type-procedure
		   type-rectype type-symbol)
      (lint-return! (@@ (srfi srfi-9) record-modifier-strict)
		    (lambda (expr)
		      [record-modifier]))))
