;; Test core functions in guile 1.8.

;; Copyright 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------
;; @

(display (@ (srfi srfi-1) first))
(display (@ (srfi srfi-1) check-arg-type))  ;; bad
(display (@ (nosuch module) first))         ;; bad

(display (@ (ice-9 q) make-q))
(use-modules (ice-9 q))
(display (@ (ice-9 q) make-q))              ;; bad (hint already in scope)
(display make-q)


;;-----------------------------------------------------------------------------
;; @@

(display (@@ (srfi srfi-1) check-arg-type))
(display (@@ (srfi srfi-1) first))          ;; bad, can use plain @
(display (@@ (srfi srfi-1) nosuchvar))      ;; bad
(display (@@ (nosuch module) first))        ;; bad


;;-----------------------------------------------------------------------------
;; begin

(begin)
(begin 1)        ;; bad (single expr)
(begin (pk 1) 2)


;;-----------------------------------------------------------------------------
;; c32 vectors

(make-c32vector -1)    ;; bad
(make-c32vector 1)
(make-c32vector 1 123)
(make-c32vector 1 'x)  ;; bad

(c32vector -1 0 1)
(c32vector 'x)         ;; bad
(c32vector 1.0)
(c32vector 1.0+1.0i)

(c32vector? 'x)

(c32vector-length 'x)                      ;; bad
(c32vector-length (c32vector 1 2 3))

(pk (c32vector-ref (c32vector 1 2 3) 2))
(c32vector-ref (c32vector 1 2 3) -1)        ;; bad

(c32vector-set! (c32vector 1 2 3) 2 123)
(c32vector-set! (c32vector 1 2 3) 2 'x)     ;; bad
(c32vector-set! (c32vector 1 2 3) -1 123)   ;; bad

(pk (c32vector->list (c32vector 1 2 3)))
(c32vector->list 'x)                       ;; bad

(pk (list->c32vector '(1 2 3)))
(list->c32vector (c32vector 1 2 3))         ;; bad
(list->c32vector (vector 1 2 3))           ;; bad
(list->c32vector 'x)                       ;; bad

(pk (any->c32vector '(1 2 3)))
(pk (any->c32vector (c32vector 1 2 3)))
(pk (any->c32vector (vector 1 2 3)))
(any->c32vector 'x)                        ;; bad


;;-----------------------------------------------------------------------------
;; call-with-blocked-asyncs

(pk call-with-blocked-asyncs)
(call-with-blocked-asyncs)                    ;; bad
(call-with-blocked-asyncs (lambda () #f))
(call-with-blocked-asyncs (lambda () #f) 123) ;; bad

(call-with-blocked-asyncs (lambda () #f))
(call-with-blocked-asyncs (lambda (x) #f))    ;; bad


;;-----------------------------------------------------------------------------
;; call-with-unblocked-asyncs

(pk call-with-unblocked-asyncs)
(call-with-unblocked-asyncs)                    ;; bad
(call-with-unblocked-asyncs (lambda () #f))
(call-with-unblocked-asyncs (lambda () #f) 123) ;; bad

(call-with-unblocked-asyncs (lambda () #f))
(call-with-unblocked-asyncs (lambda (x) #f))    ;; bad


;;-----------------------------------------------------------------------------
;; define-module

(define-module (define-module-test1)
  #:duplicates (last))

(define-module (define-module-test2)
  #:duplicates (nosuchhandlername))    ;; bad

(define-module (define-module-test3)
  #:duplicates (last nosuchhandlername))    ;; bad

(define-module (define-module-test4)   ;; bad
  #:nosuchoption
  #:export (define-module-test4-def))
(define define-module-test4-def #f)

(define-module (define-module-test5)   ;; bad
  #:nosuchoption (some thing)
  #:export (define-module-test5-def))
(define define-module-test5-def #f)


;;-----------------------------------------------------------------------------
;; eq?

;; literal arrays
(eq? #2((1 2 3) (4 5 6)) noop)    ;; bad
(eq? noop #2((1 2 3) (4 5 6)))    ;; bad


;;-----------------------------------------------------------------------------
;; eqv?

;; literal arrays
(eqv? #2((1 2 3) (4 5 6)) noop)   ;; bad
(eqv? noop #2((1 2 3) (4 5 6)))   ;; bad


;;-----------------------------------------------------------------------------
;; future

;; disabled in 1.8 for now
;; (future)         ;; not bad
;; (future 123)
;; (future 123 456) ;; not bad
;; (future nodef)   ;; not bad
;; (let ((x 1))
;;   (future x))


;;-----------------------------------------------------------------------------
;; gettext

(pk gettext)
(gettext)                              ;; bad
(pk (gettext "hello"))
(gettext 999)                          ;; bad
(pk (gettext "hello" "domain"))
(gettext "hello" 999)                  ;; bad
(pk (gettext "hello" "domain" LC_MESSAGES))
(gettext "hello" "domain" -123)        ;; bad


;;-----------------------------------------------------------------------------
;; hash-for-each

(define hash-for-each-table1 (make-hash-table 17))
(hash-for-each (lambda () #f) hash-for-each-table1)          ;; bad
(hash-for-each (lambda (key) #f) hash-for-each-table1)       ;; bad
(hash-for-each (lambda (key val) #f) hash-for-each-table1)   
(hash-for-each (lambda (key val x) #f) hash-for-each-table1) ;; bad


;;-----------------------------------------------------------------------------
;; hash-for-each-handle

(define hash-for-each-handle-table1 (make-hash-table 17))
(hash-for-each-handle (lambda () #f) hash-for-each-handle-table1)       ;; bad
(hash-for-each-handle (lambda (key) #f) hash-for-each-handle-table1)    ;; bad
(hash-for-each-handle (lambda (key val) #f) hash-for-each-handle-table1)   
(hash-for-each-handle (lambda (key val x) #f)                           ;; bad
		      hash-for-each-handle-table1)


;;-----------------------------------------------------------------------------
;; hash-map->list

(define hash-map->list-table1 (make-hash-table 17))
(hash-map->list (lambda () #f) hash-map->list-table1)          ;; bad
(hash-map->list (lambda (key) #f) hash-map->list-table1)       ;; bad
(hash-map->list (lambda (key val) #f) hash-map->list-table1)
(hash-map->list (lambda (key val x) #f) hash-map->list-table1) ;; bad


;;-----------------------------------------------------------------------------
;; integer-expt

(pk (integer-expt 10.0 2))
(pk (integer-expt 10.0 -2))


;;-----------------------------------------------------------------------------
;; make-guardian

(pk make-guardian)
(pk (make-guardian))
(make-guardian #t)                   ;; bad
(make-guardian #t 99)                ;; bad

(define make-guardian-test1 (make-guardian))
(make-guardian-test1)
(make-guardian-test1 (cons 1 2))
(make-guardian-test1 (cons 1 2) 99)  ;; bad


;;-----------------------------------------------------------------------------
;; s8 vectors

(pk (any->s8vector '(1 2 3)))
(pk (any->s8vector (s8vector 1 2 3)))
(pk (any->s8vector (vector 1 2 3)))
(any->s8vector 'x)                        ;; bad


;;-----------------------------------------------------------------------------
;; with-fluid*

(define with-fluid*-test1 (make-fluid))

(pk with-fluid*)
(with-fluid*)                                           ;; bad
(with-fluid* with-fluid*-test1)                         ;; bad
(with-fluid* with-fluid*-test1 123)                     ;; bad
(with-fluid* with-fluid*-test1 123 (lambda () #f))
(with-fluid* with-fluid*-test1 123 (lambda () #f) 123)  ;; bad

(with-fluid* with-fluid*-test1 123 (lambda (x) #f))     ;; bad


