;; Test core functions.

;; Copyright 2003, 2004, 2005, 2006, 2007, 2008 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.



;;-----------------------------------------------------------------------------
;; misc

)     ;; bad
)     ;; bad
))))  ;; bad

<unknown-procedure>       ;; bad
(<unknown-procedure> 123) ;; bad

car   ;; bad (no effect)


;;-----------------------------------------------------------------------------
;; references

(pk car)

(define reference-test1
  (let ((apply apply))
    (lambda objs (apply noop objs))))
(pk reference-test1)


;;-----------------------------------------------------------------------------
;; calls

(123 1 2)                 ;; bad
(123                      ;; bad
 (123 1 2))               ;; bad

((lambda (x) #f)
 123)

((lambda (x y) #f)        ;; bad
 123)

(#(1 2 3) 1 2)            ;; bad


;;-----------------------------------------------------------------------------
;; arities

(pk (car '(1 2 3)))
(car)                     ;; bad

(pk (noop car cdr))


;;-----------------------------------------------------------------------------
;; module use

(use-module (ice-9 q))    ;; bad

(use-modules (ice-9 q))
(use-modules (ice-9 q))   ;; bad
(pk enq!) ;; module used

(use-modules (ice-9 optargs))
(define* (module-use-test1 #:optional bar) #f)
(pk module-use-test1)

(define deq! 123)         ;; bad, shadowing import
(pk deq!) ;; define used


;;-----------------------------------------------------------------------------
;; <

(<)                       ;; bad
(< 1)                     ;; bad
(pk (< 1 2))


;;-----------------------------------------------------------------------------
;; <=

(<=)                      ;; bad
(<= 1)                    ;; bad
(pk (<= 1 2))


;;-----------------------------------------------------------------------------
;; =

(=)                       ;; bad
(= 1)                     ;; bad
(pk (= 1 2))


;;-----------------------------------------------------------------------------
;; >

(>)                       ;; bad
(> 1)                     ;; bad
(pk (> 1 2))


;;-----------------------------------------------------------------------------
;; >=

(>=)                      ;; bad
(>= 1)                    ;; bad
(pk (>= 1 2))


;;-----------------------------------------------------------------------------
;; /

(pk /)
(pk (/))                  ;; bad
(pk (/ 5))
(pk (/ 5 7))
(pk (/ 1 7))              ;; bad (hint about `/ 1')


;;-----------------------------------------------------------------------------
;; and

(pk and)
(and)                     ;; bad
(and 1)                   ;; bad
(and 1 2)
(and nodef #f)            ;; bad
(and #f
     nodef)
(and (defined? 'nodef)
     (nodef))


;;-----------------------------------------------------------------------------
;; append

(pk (append))
(pk (append 123))
(pk (append 123 456))          ;; bad
(pk (append (list 123) 456))
(pk (append '()))
(pk (append '() '()))
(pk (append '() '(456)))
(pk (append '(456) '()))


;;-----------------------------------------------------------------------------
;; apply

(apply)                             ;; bad
(apply noop)                        ;; bad
(apply noop '())
(apply noop 1 2 3 '())
(apply noop 'x)                     ;; bad
(apply noop '() 'x)                 ;; bad
(apply noop '() '() 'x)             ;; bad

(apply log 1 '())
(apply log 1 2 '())                 ;; bad
(apply log 1 2 3 '())               ;; bad

(define (apply-test1 x) #f)
(apply apply-test1 '())
(apply apply-test1 1 '())
(apply apply-test1 1 2 '())         ;; bad
(apply apply-test1 1 2 3 '())       ;; bad

(define (apply-test2 x y) #f)
(apply apply-test2 '())
(apply apply-test2 1 '())
(apply apply-test2 1 2 '())
(apply apply-test2 1 2 3 '())       ;; bad

(define (apply-test3)
  (apply apply-test3func '())
  (apply apply-test3func 1 '())
  (apply apply-test3func 1 2 '()))  ;; bad
(define (apply-test3func x) #f)
(pk apply-test3)

(apply nodef 1 2 3 '())             ;; bad

(define (apply-test4)
  (apply nodef 1 2 3 '()))          ;; bad
(pk apply-test4)

(apply append '((1) (2)))

(apply car 'foo '())                ;; bad
(apply car '(1 . 2) '())
(apply simple-format '(#t "~a" 1))
(apply simple-format #f '("~a" 1))  ;; notyet bad (called for effect)
(apply simple-format #t '("~a" 1))
(apply simple-format #t "~a" '(1))
(apply simple-format #t "~a" 1 '())
(apply simple-format 'xx "~a" '(1)) ;; bad


;;-----------------------------------------------------------------------------
;; array-for-each

(pk array-for-each)
(array-for-each)                                   ;; bad
(array-for-each noop)                              ;; bad
(array-for-each noop #1(3 4))
(array-for-each noop #1(3 4) #1(5 6))

(array-for-each (lambda () #f) #1(3 4))             ;; bad
(array-for-each (lambda (x) #f) #1(3 4))
(array-for-each (lambda (x y) #f) #1(3 4))          ;; bad
(array-for-each (lambda (x y z) #f) #1(3 4))        ;; bad

(array-for-each (lambda () #f) #1(3 4) #1(5 6))      ;; bad
(array-for-each (lambda (x) #f) #1(3 4) #1(5 6))     ;; bad
(array-for-each (lambda (x y) #f) #1(3 4) #1(5 6))
(array-for-each (lambda (x y z) #f) #1(3 4) #1(5 6)) ;; bad


;;-----------------------------------------------------------------------------
;; array-map!

(pk array-map!)
(array-map!)                         ;; bad
(array-map! #1(1 2))                 ;; bad
(array-map! #1(1 2) noop)
(array-map! #1(1 2) noop #1(3 4))
(array-map! #1(1 2) noop #1(3 4) #1(5 6))

(array-map! #1(1 2) (lambda () #f) #1(3 4))              ;; bad
(array-map! #1(1 2) (lambda (x) #f) #1(3 4))
(array-map! #1(1 2) (lambda (x y) #f) #1(3 4))           ;; bad
(array-map! #1(1 2) (lambda (x y z) #f) #1(3 4))         ;; bad

(array-map! #1(1 2) (lambda () #f) #1(3 4) #1(5 6))      ;; bad
(array-map! #1(1 2) (lambda (x) #f) #1(3 4) #1(5 6))     ;; bad
(array-map! #1(1 2) (lambda (x y) #f) #1(3 4) #1(5 6))
(array-map! #1(1 2) (lambda (x y z) #f) #1(3 4) #1(5 6)) ;; bad


;;-----------------------------------------------------------------------------
;; array-map-in-order!

(pk array-map-in-order!)
(array-map-in-order!)                         ;; bad
(array-map-in-order! #1(1 2))                 ;; bad
(array-map-in-order! #1(1 2) noop)            ;; bad
(array-map-in-order! #1(1 2) noop #1(3 4))
(array-map-in-order! #1(1 2) noop #1(3 4) #1(5 6))

(array-map-in-order! #1(1 2) (lambda () #f) #1(3 4))              ;; bad
(array-map-in-order! #1(1 2) (lambda (x) #f) #1(3 4))
(array-map-in-order! #1(1 2) (lambda (x y) #f) #1(3 4))           ;; bad
(array-map-in-order! #1(1 2) (lambda (x y z) #f) #1(3 4))         ;; bad

(array-map-in-order! #1(1 2) (lambda () #f) #1(3 4) #1(5 6))      ;; bad
(array-map-in-order! #1(1 2) (lambda (x) #f) #1(3 4) #1(5 6))     ;; bad
(array-map-in-order! #1(1 2) (lambda (x y) #f) #1(3 4) #1(5 6))
(array-map-in-order! #1(1 2) (lambda (x y z) #f) #1(3 4) #1(5 6)) ;; bad


;;-----------------------------------------------------------------------------
;; assq

(pk (assq #f (list 1 2 3)))
(assq "x" (list 1 2 3))          ;; bad


;;-----------------------------------------------------------------------------
;; assq-ref

(pk (assq-ref (list 1 2 3) #f))
(assq-ref (list 1 2 3) "x")      ;; bad


;;-----------------------------------------------------------------------------
;; assq-remove!

(assq-remove! (list 1 2 3) #f)
(assq-remove! (list 1 2 3) "x")  ;; bad

;;-----------------------------------------------------------------------------
;; assq-set!

(assq-set! (list 1 2 3) #f 99)
(assq-set! (list 1 2 3) #f nodef) ;; bad
(assq-set! (list 1 2 3) "x" 99)   ;; bad


;;-----------------------------------------------------------------------------
;; assv

(pk (assv #f (list 1 2 3)))
(assv "x" (list 1 2 3))          ;; bad


;;-----------------------------------------------------------------------------
;; assv-ref

(pk (assv-ref (list 1 2 3) #f))
(assv-ref (list 1 2 3) "x")      ;; bad


;;-----------------------------------------------------------------------------
;; assv-remove!

(assv-remove! (list 1 2 3) #f)
(assv-remove! (list 1 2 3) "x")  ;; bad


;;-----------------------------------------------------------------------------
;; assv-set!

(assv-set! (list 1 2 3) #f 99)
(assv-set! (list 1 2 3) #f nodef) ;; bad
(assv-set! (list 1 2 3) "x" 99)   ;; bad


;;-----------------------------------------------------------------------------
;; atan

(pk atan)
(pk (atan 0))
(pk (atan 1 2))
(atan 1 2 3)              ;; bad


;;-----------------------------------------------------------------------------
;; call-with-current-continuation

(pk call-with-current-continuation)
(call-with-current-continuation)                     ;; bad
(call-with-current-continuation (lambda (c) #f))
(call-with-current-continuation (lambda (c) #f) 123) ;; bad

(call-with-current-continuation (lambda () #f))      ;; bad
(call-with-current-continuation (lambda (c) #f))
(call-with-current-continuation (lambda (c x) #f))   ;; bad


;;-----------------------------------------------------------------------------
;; call-with-dynamic-root

(pk call-with-dynamic-root)
(call-with-dynamic-root)                                              ;; bad
(call-with-dynamic-root (lambda () #f))                               ;; bad
(call-with-dynamic-root (lambda () #f) (lambda (key . args) #f))
(call-with-dynamic-root (lambda () #f) (lambda (key . args) #f) 123)  ;; bad

(call-with-dynamic-root (lambda () #f) (lambda (key . args) #f))
(call-with-dynamic-root (lambda (x) #f) (lambda (key . args) #f))     ;; bad

(call-with-dynamic-root (lambda () #f) (lambda () #f))                ;; bad
(call-with-dynamic-root (lambda () #f) (lambda (key . args) #f))


;;-----------------------------------------------------------------------------
;; call-with-input-file

(pk call-with-input-file)
(call-with-input-file)                                   ;; bad
(call-with-input-file "/etc/passwd")                     ;; bad
(call-with-input-file "/etc/passwd" (lambda (p) #f))
(call-with-input-file "/etc/passwd" (lambda (p) #f) 123) ;; bad

(call-with-input-file "/etc/passwd" (lambda () #f))      ;; bad
(call-with-input-file "/etc/passwd" (lambda (p) #f))
(call-with-input-file "/etc/passwd" (lambda (p x) #f))   ;; bad


;;-----------------------------------------------------------------------------
;; call-with-input-string

(pk call-with-input-string)
(call-with-input-string)                           ;; bad
(call-with-input-string "foo")                     ;; bad
(call-with-input-string "foo" (lambda (p) #f))
(call-with-input-string "foo" (lambda (p) #f) 123) ;; bad

(call-with-input-string "foo" (lambda () #f))      ;; bad
(call-with-input-string "foo" (lambda (p) #f))
(call-with-input-string "foo" (lambda (p x) #f))   ;; bad


;;-----------------------------------------------------------------------------
;; call-with-output-file

(pk call-with-output-file)
(call-with-output-file)                                   ;; bad
(call-with-output-file "/etc/passwd")                     ;; bad
(call-with-output-file "/etc/passwd" (lambda (p) #f))
(call-with-output-file "/etc/passwd" (lambda (p) #f) 123) ;; bad

(call-with-output-file "/etc/passwd" (lambda () #f))      ;; bad
(call-with-output-file "/etc/passwd" (lambda (p) #f))
(call-with-output-file "/etc/passwd" (lambda (p x) #f))   ;; bad


;;-----------------------------------------------------------------------------
;; call-with-output-string

(pk call-with-output-string)
(call-with-output-string)                     ;; bad
(call-with-output-string (lambda (p) #f))
(call-with-output-string (lambda (p) #f) 123) ;; bad

(call-with-output-string (lambda () #f))      ;; bad
(call-with-output-string (lambda (p) #f))
(call-with-output-string (lambda (p x) #f))   ;; bad


;;-----------------------------------------------------------------------------
;; call-with-values

(pk call-with-values)
(call-with-values)                                                ;; bad
(call-with-values (lambda () (values 1 2)))                       ;; bad
(call-with-values (lambda () (values 1 2)) (lambda (x y) #f))
(call-with-values (lambda () (values 1 2)) (lambda (x y) #f) 123) ;; bad
(call-with-values (lambda (p) #f) 123) ;; bad

(call-with-values (lambda () (values 1 2)) (lambda (x y) #f))
(call-with-values (lambda (x) (values 1 2)) (lambda (x y) #f))    ;; bad


;;-----------------------------------------------------------------------------
;; car

(pk car)
(pk (car))                     ;; bad
(pk (car '(x . y)))
(pk (car '(x . y) '(x . y)))   ;; bad
(pk (car 999))                 ;; bad
(car (cons 'x 'y))             ;; bad (called for effect)
(pk (car (cons 'x 'y)))


;;-----------------------------------------------------------------------------
;; case

(case)                    ;; bad
(case 'key)               ;; bad
(case 'key 'x)            ;; bad

(case 'x ((x)))           ;; bad
(case 'x
  ((x))                   ;; bad
  ((y)))                  ;; bad

(case 'x
  ((x) #f)
  ((x) #f))               ;; bad

(case 'key
  ((1 2 3)
   #f)
  ((1 2 3)                ;; bad
   #f))

(case 'key
  (("x" "y")              ;; bad
   #f))

(case 'x
  (else                   ;; bad
   (nodef))               ;; bad
  ((x) 2))

(case 'x
  ((x) 2)
  (else
   (nodef)))              ;; bad

(case 'x
  (((123))         	  ;; bad
   (nodef)))       	  ;; bad

(case 'x
  ((#(1 2 3))      	  ;; bad
   (nodef)))       	  ;; bad

(case 'x
  ((#2((99) (99)))     	  ;; bad
   (nodef)))       	  ;; bad


;-----------------------------------------------------------------------------
;; cond

(pk cond)
(cond)                      ;; bad
(cond ())                   ;; bad
(cond ('x 'y) ())           ;; bad
(cond ('x 'y) ('z 'z))
(cond ((noop) 'x 'y 'z))
(cond (#t => (lambda (x) #f)))
(cond (else))               ;; bad
(cond (else #f))
(cond (else =>))            ;; bad
(cond (else => 123))        ;; bad
(cond (else => noop))       ;; bad

(cond (#f 123)
      (else #f))
(cond (else #f)             ;; bad
      (#f 123))

(define cond-test1 #f)
(cond (#t => (lambda (x) cond-test1)))


;;-----------------------------------------------------------------------------
;; cond-expand

(pk cond-expand)
(cond-expand)               ;; bad
(cond-expand . 1)           ;; bad
(cond-expand 1)             ;; bad
(cond-expand (guile 1)
	     )
(cond-expand ((blah blah)   ;; bad
	      )
	     ((and)         ;; bad
	      )
	     ((and x . y)   ;; bad
	      1)
	     ((and xyz)
	      )
	     ((and x y)
	      1)
	     ((not . guile) ;; bad
	      1)
	     ((not guile)
	      1)
	     ((not x y)     ;; bad
	      1)
	     ((or)          ;; bad
	      )
	     ((or x . y)    ;; bad
	      1)
	     ((or xyz)
	      )
	     ((or x y)
	      1))

(cond-expand (guile
	      (nodef))     ;; bad
	     (not-feature
	      (nodef)))

(cond-expand ((not (not guile))
	      (nodef))     ;; bad
	     (not-feature
	      (nodef)))

(cond-expand               ;; bad
 ((not guile)
  (nodef))
 (not-feature
  (nodef)))

(cond-expand ((and guile)
	      (nodef))     ;; bad
	     (not-feature
	      (nodef)))

(cond-expand ((and guile nosuch)
	      (nodef))
	     (else
	      (nodef)))    ;; bad

(cond-expand ((or guile)
	      (nodef))     ;; bad
	     (not-feature
	      (nodef)))

(cond-expand ((or guile)
	      (nodef))     ;; bad
	     (not-feature
	      (nodef)))

(cond-expand ((or nosuch guile)
	      (nodef))     ;; bad
	     (else
	      (nodef)))

(cond-expand ((or nosuch (and guile (not (not guile))))
	      (nodef))     ;; bad
	     (else
	      (nodef)))

(cond-expand ((or nosuch else)
	      (nodef))
	     (else
	      (nodef)))    ;; bad


;;-----------------------------------------------------------------------------
;; define

define-test1              ;; bad
(define-test2)            ;; bad

(define define-test3 123)
(define-test3)
(pk define-test3)

(define define-test4)                ;; bad

(define (define-test5 b b) #f)       ;; bad
(pk define-test5)

(define (define-test5b define-test5b) #f)
(pk define-test5b)

(define ((define-test6) c c) #f)     ;; bad
(pk define-test6)

(define ((define-test7 d) d) #f)     ;; bad
(pk define-test7)

(define
  (((define-test8) e e) f)           ;; bad
  #f)
(pk define-test8)

(define define-test9 (lambda (x) #f))
(define-test9 1 2 3)                 ;; bad

(define (define-test9)
  (define x 1)
  (pk (noop))
  x)
(pk define-test9)

(define (define-test10)
  (define (x) y)
  (define (y) x)
  123)
(pk define-test10)

(define (define-test11)
  (begin
    (define foo 1))
  foo)
(pk define-test11)

(define (define-test12)
  (begin
    (define foo 1)
    (pk foo))
  123)
(pk define-test12)

(define (define-test13)
  (let ((x (lambda (arg) #f)))  ;; bad (unused)
    (define (y) (x))
    (define (x) (y))
    123))

(define (define-test13)
  (let ((x (lambda (arg) 1)))   ;; bad (unused)
    (define y x)
    (define (x) 123)
    (y)                         ;; bad (args)
    x))
(pk define-test13)

(define (define-test14)
  "docstring"
  (define (x) #f)
  (x))
(pk define-test14)

(define (define-test15)
  "docstring"
  (define (x) 123)
  (define (y) 456)
  (+ (x) (y)))
(pk define-test15)

(define (define-test16)
  "docstring"
  (define (x) 123)
  (pk "another string")
  (define (y) 456)              ;; bad
  (+ (x) (y)))
(pk define-test16)

;; repeat binding just does `set!'
(define (define-test17)
  #f)
(define (define-test17)
  #f)
(pk define-test17)

(define (define-test18)         ;; bad (never used)
  #f)

(define ->bool #f)              ;; bad (shadow imported)
(pk ->bool)

(define (symbol-append) #f)     ;; bad (shadow imported)
(pk symbol-append)

(define (define-test19)
  (define x 123)
  (define x 456)                ;; bad (duplicate internal define)
  x)
(pk define-test19)

(define (define-test20 lst)     ;; bad, unused
  (if (null? lst)
      #f
      (define-test20 (cdr lst))))

(define define-test21           ;; bad, unused
  (lambda (x)
    (if (positive? x)
	(define-test21 (1- x))
	(define-test21 (1+ x)))))


;;-----------------------------------------------------------------------------
;; define-public

(let ()
  (define-public define-public-test1 1))    ;; bad

(define-public (define-test14)
  "docstring"
  (define (x) #f)
  (x))

(define-public (define-test15)
  "docstring"
  (define (x) 123)
  (define (y) 456)
  (+ (x) (y)))

(define-public (define-test16)
  "docstring"
  (define (x) 123)
  (pk "another string")
  (define (y) 456)                          ;; bad
  (+ (x) (y)))



;;-----------------------------------------------------------------------------
;; defmacro

(defmacro defmacro-test1)     ;; bad
(defmacro defmacro-test2 (x)) ;; bad
(defmacro "dm" (x) `(noop))   ;; bad
(defmacro defmacro-test3 123  ;; bad
  `(noop))

(defmacro defmacro-test4 (x) x)
(defmacro-test4 1)
(defmacro-test4 nodef)        ;; bad
(defmacro-test4 1 2)          ;; bad

(defmacro defmacro-test5 (x)
  (display x (%make-void-port "w"))
  `(noop))
(defmacro-test5 1)
(defmacro-test5 nodef)
(defmacro-test5 1 2)          ;; bad

(defmacro defmacro-test6 (x . rest)
  `(noop ,x ,rest))
(pk defmacro-test6)

(defmacro defmacro-test7 args
  (pk args)
  `(noop))
(pk defmacro-test7)

(defmacro defmacro-test8 (x y)
  (pk x)
  (pk y)
  `(begin
     (define (exp) #f)
     (display exp)))
(pk defmacro-test8)

(defmacro defmacro-test9 (x y)
  `(+ ,x ,y))
(defmacro-test9 123 456)
(defmacro-test9 123 nodef)    ;; bad

(defmacro defmacro-test10 (x)
  (define internal #f)
  (internal)
  (pk x)
  '(noop))
(pk defmacro-test10)

(defmacro defmacro-test11 (cond . body)
  `(if ,cond (begin ,@body)))
(cond
 (else
  (noop)))
(pk defmacro-test11)

(defmacro defmacro-test12 (x) (nodef))  ;; bad
(defmacro-test12 1)                     ;; bad


;;-----------------------------------------------------------------------------
;; define-macro

(define-macro (define-macro-test1 x)
  (define internal #f)
  (internal)
  (pk x)
  '(noop))
(pk define-macro-test1)

(define-macro (define-macro-test2 cond . body)
  `(if ,cond (begin ,@body)))
(cond
 (else
  (noop)))
(pk define-macro-test2)

(define-macro (define-macro-test3 x)
  "this is a docstring"
  (define internal #f)
  (internal)
  (pk x)
  '(noop))
(pk define-macro-test3)

(define-macro (define-macro-test4 x)
  "this is a result")
(pk define-macro-test4)



;;-----------------------------------------------------------------------------
;; define-module

receive                    ;; bad

(define-module (define-module-test0)
  #:use-module (ice-9 receive))
(pk receive)

(define-module (define-module-test0b)
  :use-module (ice-9 popen))
(pk open-input-pipe)

(define-module)            ;; bad

(define-module (define-module-test1)    ;; bad
  #:use-module)

(define-module (define-module-test2)    ;; bad
  #:autoload)

(define-module (define-module-test3)    ;; bad
  #:autoload (some module))

(define-module (define-module-test4)
  #:export (1 2 3))           ;; bad

(define-module (define-module-test5)
  #:export (define-module-test5-nodef)) ;; bad

(define-module (define-module-test6)
  #:export (define-module-test6-var))
(define define-module-test6-var #f)

(define-module (define-module-test6b)
  :use-module (ice-9 and-let-star)
  :export (define-module-test6-var))
(define define-module-test6-var #f)
(pk and-let*)

(let ()
  (define-module (foo bar))             ;; bad
  #f)


;;-----------------------------------------------------------------------------
;; defined?

(pk defined?)
(defined?)                ;; bad
(pk (defined? 'x))
(pk (defined? 'x 'env))
(defined? 'x 'env 'blah)  ;; bad


;;-----------------------------------------------------------------------------
;; delay

(delay car)

(define delay-test1 (delay delay-test1-sym))
(define delay-test1-sym 123)
(pk delay-test1)

(let ()
  (define delay-test2 (delay delay-test2-sym))
  (define delay-test2-sym 123)
  delay-test2)

(let ()
  (define delay-test3
    (delay delay-test3-undef)) ;; bad
  delay-test3)


;;-----------------------------------------------------------------------------
;; delq

(pk (delq #f (list 1 2 3)))
(delq "x" (list 1 2 3))       ;; bad


;;-----------------------------------------------------------------------------
;; delq!

(delq! #f (list 1 2 3))
(delq! "x" (list 1 2 3))      ;; bad


;;-----------------------------------------------------------------------------
;; delq1!

(delq1! #f (list 1 2 3))
(delq1! "x" (list 1 2 3))     ;; bad


;;-----------------------------------------------------------------------------
;; delv

(pk (delv #f (list 1 2 3)))
(delv "x" (list 1 2 3))       ;; bad


;;-----------------------------------------------------------------------------
;; delv!

(delv! #f (list 1 2 3))
(delv! "x" (list 1 2 3))      ;; bad


;;-----------------------------------------------------------------------------
;; delv1!

(delv1! #f (list 1 2 3))
(delv1! "x" (list 1 2 3))     ;; bad


;;-----------------------------------------------------------------------------
;; do

(do)                      ;; bad
(do #f)                   ;; bad
(do (123))                ;; bad
(do () 123)               ;; bad
(do () (123))
(do () (#t) #f)
(do () ((pk #t) (pk #t) #t)
  #f)

(do ((x 1 (1+ x))
     (x 1 (1+ x)))        ;; bad
    (#t)
  #f)


;;-----------------------------------------------------------------------------
;; dup

(pk dup)
(dup)                              ;; bad
(dup (current-output-port))
(dup (current-output-port) 1)
(dup (current-output-port) 1 999)  ;; bad


;;-----------------------------------------------------------------------------
;; dup->inport

(pk dup->inport)
(dup->inport)                               ;; bad
(dup->inport (current-input-port))
(dup->inport (current-input-port) 0)
(dup->inport (current-input-port) 0 999)    ;; bad


;;-----------------------------------------------------------------------------
;; dup->outport

(pk dup->outport)
(dup->outport)                              ;; bad
(dup->outport (current-output-port))
(dup->outport (current-output-port) 1)
(dup->outport (current-output-port) 1 999)  ;; bad


;;-----------------------------------------------------------------------------
;; dup->port

(pk dup->port)
(dup->port)                                 ;; bad
(dup->port (current-input-port))            ;; bad
(dup->port (current-input-port) "r")
(dup->port (current-input-port) "r" 0)
(dup->port (current-input-port) "r" 0 999)  ;; bad


;;-----------------------------------------------------------------------------
;; duplicate-port

(pk duplicate-port)
(duplicate-port)                            ;; bad
(duplicate-port (current-input-port))       ;; bad
(duplicate-port (current-input-port) "r")
(duplicate-port (current-input-port) "xyz") ;; bad


;;-----------------------------------------------------------------------------
;; dynamic-wind

(pk dynamic-wind)
(dynamic-wind)                        	    ;; bad
(dynamic-wind noop)                   	    ;; bad
(dynamic-wind noop noop)              	    ;; bad
(dynamic-wind noop noop noop)

(dynamic-wind (lambda (x) #f) noop noop)    ;; bad
(dynamic-wind noop (lambda (x) #f) noop)    ;; bad
(dynamic-wind noop noop (lambda (x) #f))    ;; bad


;;-----------------------------------------------------------------------------
;; eq?

(eq?)                             ;; bad
(eq? #t)                          ;; bad
(pk (eq? #t #t))

;; bools
(pk (eq? #t car))
(pk (eq? #f car))

;; inums
(pk (eq? 0 car))
(pk (eq? 99 car))
(pk (eq? -99 car))
(pk (eq? car 99))
(pk (eq? car -99))

;; chars
(pk (eq? #\a car))
(pk (eq? car #\a))

;; bignums
(eq? 9999999999999999999999 car)  ;; bad
(eq? -9999999999999999999999 car) ;; bad

;; flonums
(eq? 1.0 car)                     ;; bad
(eq? car 1.0)                     ;; bad

;; fractions (flonums in 1.6, also bad)
(eq? 1/2 car)                     ;; bad
(eq? car 1/2)                     ;; bad

;; literal vectors
(eq? #(1 2 3) car)                ;; bad
(eq? '#(1 2 3) car)               ;; bad
(eq? car #(1 2 3))                ;; bad
(eq? car '#(1 2 3))               ;; bad

;; literal lists
(eq? '(1 2) car)                  ;; bad
(eq? '(1 . 2) car)                ;; bad
(eq? car '(1 . 2))                ;; bad

;; literal strings
(eq? "foo" car)                   ;; bad
(eq? car "foo")                   ;; bad


;;-----------------------------------------------------------------------------
;; eqv?

(eqv?)                            ;; bad
(eqv? #t)                         ;; bad
(pk (eqv? #t #t))

;; literal vectors
(eqv? #(1 2 3) car)               ;; bad
(eqv? '#(1 2 3) car)              ;; bad
(eqv? car #(1 2 3))               ;; bad
(eqv? car '#(1 2 3))              ;; bad

;; literal lists
(eqv? '(1 2) car)                 ;; bad
(eqv? '(1 . 2) car)               ;; bad
(eqv? car '(1 . 2))               ;; bad

;; literal strings
(eqv? "foo" car)                  ;; bad
(eqv? car "foo")                  ;; bad


;;-----------------------------------------------------------------------------
;; equal?

(equal?)                          ;; bad
(equal? #t)                       ;; bad
(pk (equal? #t #t))


;;-----------------------------------------------------------------------------
;; error

(pk error)
(error)
(error "hello")
(error "hello" 123)
(error "hello ~a" 123)            ;; bad

(error 'hello 'world)


;;-----------------------------------------------------------------------------
;; export

(let ((x 1))
  (export x))              ;; bad

(export export-test1nodef) ;; bad

(define export-test2 #f)
(export export-test2)

(export export-test3)
(define export-test3 #f)


;;-----------------------------------------------------------------------------
;; flock

(flock (current-output-port) LOCK_SH)
(flock (current-output-port) LOCK_EX)
(flock (current-output-port) (logior LOCK_EX LOCK_NB))
(flock (current-output-port) LOCK_UN)
(flock (current-output-port) -1)  ;; bad


;;-----------------------------------------------------------------------------
;; getenv

(getenv)              ;; bad
(getenv #f)           ;; bad
(getenv 123)          ;; bad
(getenv 'x)           ;; bad
(pk (getenv "FOO"))
(getenv "FOO" "BAR")  ;; bad
(- (getenv "FOO"))    ;; bad


;;-----------------------------------------------------------------------------
;; getpgrp

(pk (getpgrp))
(getpgrp)             ;; bad (pure)
(getpgrp 123)         ;; bad


;;-----------------------------------------------------------------------------
;; getpid

(pk (getpid))
(getpid)              ;; bad (pure)
(getpid 123)          ;; bad


;;-----------------------------------------------------------------------------
;; getpriority

(pk (getpriority PRIO_PROCESS 1))
(pk (getpriority PRIO_PGRP    1))
(pk (getpriority PRIO_USER    0))
(pk (getpriority -123 1))    ;; bad


;;-----------------------------------------------------------------------------
;; integer-expt

(integer-expt)          ;; bad
(integer-expt 10)       ;; bad
(pk (integer-expt 10 2))
(integer-expt 10 2 5)   ;; bad

(pk (integer-expt 10.0 2))
(integer-expt 10 2.0)   ;; bad


;;-----------------------------------------------------------------------------
;; false-if-exception

(false-if-exception)       ;; bad
(false-if-exception 1)
(false-if-exception 1 2)   ;; bad
(false-if-exception 1 2 3) ;; bad


;;-----------------------------------------------------------------------------
;; for-each

(pk for-each)
(for-each (lambda () #f) '(123))             ;; bad
(for-each (lambda (x) #f) '(123))
(for-each (lambda (x y) #f) '(123))          ;; bad
(for-each (lambda (x y z) #f) '(123))        ;; bad

(for-each (lambda () #f) '(123) '(456))      ;; bad
(for-each (lambda (x) #f) '(123) '(456))     ;; bad
(for-each (lambda (x y) #f) '(123) '(456))
(for-each (lambda (x y z) #f) '(123) '(456)) ;; bad

(odd? (for-each (lambda (x) #f) '(123)))     ;; badd


;;-----------------------------------------------------------------------------
;; hash-fold

(define hash-fold-table1 (make-hash-table 17))
(hash-fold)                                                        ;; bad
(hash-fold (lambda (key val prev) #f))                             ;; bad
(hash-fold (lambda (key val prev) #f) 'init)                       ;; bad
(hash-fold (lambda (key val prev) #f) 'init hash-fold-table1)
(hash-fold (lambda (key val prev) #f) 'init hash-fold-table1 123)  ;; bad

(hash-fold (lambda () #f) 'init hash-fold-table1)                  ;; bad
(hash-fold (lambda (key) #f) 'init hash-fold-table1)               ;; bad
(hash-fold (lambda (key val) #f) 'init hash-fold-table1)           ;; bad
(hash-fold (lambda (key val prev) #f) 'init hash-fold-table1)
(hash-fold (lambda (key val prev x) #f) 'init hash-fold-table1)    ;; bad


;;-----------------------------------------------------------------------------
;; if

(pk if)
(if)                      ;; bad
(if 'cond)                ;; bad
(if 'cond #t #f)
(if 'cond 1 2 3)          ;; bad

(if nodef nodef)          ;; bad

(if (defined? 'nodef)
    (nodef))

(if (defined? 'nodef)
    (nodef)
    (nodef))                 ;; bad

(if (not (defined? 'nodef))
    (nodef))                 ;; bad

(if (not (defined? 'nodef))
    (nodef)                  ;; bad
    (nodef))

(if (not (not (defined? 'nodef)))
    (nodef))

(if (not (not (defined? 'nodef)))
    (nodef) 
    (nodef))                 ;; bad

(define (if-test1)
  (if (defined? 'nodef)
      (nodef)))              ;; bad
(pk if-test1)

(define (if-test2)
  (if (defined? 'nodef)
      (nodef)                ;; bad
      (nodef)))              ;; bad
(pk if-test2)

(define (if-test3)
  (if (not (defined? 'nodef))
      (nodef)))              ;; bad
(pk if-test3)

(define (if-test4)
  (if (not (defined? 'nodef))
      (nodef)                ;; bad
      (nodef)))              ;; bad
(pk if-test4)

(if (provided? 123)                 ;; bad
    (nodef))                   	    ;; bad
(if (provided? 'delay)
    (nodef))                   	    ;; bad
(if (provided? 'nosuchfeature)
    (nodef))

(if (feature? 123)                  ;; bad
    (nodef))                   	    ;; bad
(if (feature? 'delay)
    (nodef))                   	    ;; bad
(if (feature? 'nosuchfeature)
    (nodef))

(if (or)                            ;; bad
    (nodef))
(if (or (feature? 'nosuchfeature))  ;; bad
    (nodef))
(if (or (feature? 'nosuchfeature) (feature? 'delay))
    (nodef))                        ;; bad

(if (and)                           ;; bad
    (noop)
    (nodef))
(if (and (feature? 'nosuchfeature)) ;; bad
    (nodef))
(if (and (feature? 'nosuchfeature) (feature? 'delay))
    (nodef))
(if (feature? 'complex)
    (nodef))                        ;; bad
(if (feature? 'delay)
    (nodef))                        ;; bad
(if (and (feature? 'complex)
	 (feature? 'delay))
    (nodef))                        ;; bad

(if (defined? 'hash-table?)
    hash-table?
    vector?)


;;-----------------------------------------------------------------------------
;; inexact->exact

(pk inexact->exact)
(inexact->exact)                     ;; bad
(inexact->exact "x")                 ;; bad
(inexact->exact 'x)                  ;; bad
(pk (inexact->exact 1.0))


;;-----------------------------------------------------------------------------
;; lambda

(pk (lambda () #f))
(lambda (a a) #f)         ;; bad (duplicate binding)

(lambda (1 2 3) #f)       ;; bad

(pk (lambda ()
      (define x 1)
      (pk (noop))
      x))

(pk (lambda ()
      (pk (noop))
      (define x 1)))      ;; bad


;;-----------------------------------------------------------------------------
;; last-stack-frame

(pk last-stack-frame)
(last-stack-frame)                       ;; bad
(call-with-current-continuation (lambda (cont)
				  (last-stack-frame cont)))


;;-----------------------------------------------------------------------------
;; let, including named let

(let "x"                  ;; bad
  #f)

(let (x . y)              ;; bad
  #f)

(let (x y)                ;; bad
  #f)

(let ((x 1 2))            ;; bad
  #f)

(let ((x 1)
      (x 2))              ;; bad
  #f)

(let () #f)
(let ())                  ;; bad

(let foo ((x 1)
	  (y 2))
  (list foo x y))

(let foo
    ((x foo))             ;; bad
  (pk x)
  foo)

(let foo ((x 1)
	  (y 2))
  (foo 1 2 3)             ;; bad
  (pk x)
  (pk y))

(let ((x (lambda (y) #f)))
  (x 1 2 3))              ;; bad

(let ()                   ;; bad (empty body)
  (define x 1))           ;; bad (unused)

(let foo ()
  (define x 1)
  (pk x)
  foo)

(let ()
  (pk (noop))
  (define x 1)            ;; bad
  #f)

(let ()
  (define (x)
    (y 1))                ;; bad
  (define (y)
    (x 1))                ;; bad
  #f)

(define (let-test1) #f)
(pk let-test1)
(let ()
  (define (x)
    (let-test1 1))
  (define (let-test1 y) #f)
  (pk let-test1) 
  x)

(let foo ()
  (define (x)
    (y 1))                ;; bad
  (define (y)
    (x 1))                ;; bad
  foo)

(let ()                   ;; bad
  nosuchvariable)

(define let-test-proc1
  (let ((x 1)
	(y 2))
    (pk x)
    (pk y)
    (lambda (a b c) #f)))
(let-test-proc1 1)        ;; bad

(let foo ((x (lambda ()
	       (foo 1)))) ;; bad (loop not in scope of vars)
  (pk x)
  foo)


;;-----------------------------------------------------------------------------
;; let*

(let* ((x 1) (y x))
  (define let*-test1 1)
  (pk y)
  let*-test1)
let*-test1                ;; bad

(let* ((x (lambda (arg) #f)) (y x))
  (pk y)
  (x 1)
  (x 1 2 3))              ;; bad

(let* ((x 1) (y x)) y)
(let*                     ;; bad
    ((x 1)
     (y x)))

(let* ((a 1) (b a))
  (define x 1)
  (pk x)
  (pk b))

(let* ((a 1) (b a))
  (pk a)
  (pk b)
  (pk (noop))
  (define x 1)            ;; bad
  #f)

(let* ((x 1) (zz x))
  (pk x)
  (pk zz)
  (define y 1)            ;; bad
  (pk y))

(define let*-test-proc1
  (let* ((x 1)
	 (y x))
    (pk x)
    (pk y)
    (lambda (a b c) #f)))
(let*-test-proc1 1)        ;; bad

(let* ((x 1)               ;; not bad yet (can be plain let)
       (y 2))
  (pk x)
  (pk y))
(let* ((x 1) (y x))
  (pk x)
  (pk y))
(let* ((x 1) (y 2) (z x))
  (pk x)
  (pk y)
  (pk z))
(let* ((x 1) (y 2) (z y))
  (pk x)
  (pk y)
  (pk z))
(let* ((x 1) (y 2) (z (lambda () x)))
  (pk x)
  (pk y)
  (pk z))
(let* ((x 1) (y 2) (z (lambda () y)))
  (pk x)
  (pk y)
  (pk z))


;;-----------------------------------------------------------------------------
;; letrec

(letrec ((x 1)
	 (x 2))              ;; bad
  #f)

(letrec ((x (lambda (y) #f)))
  (x 1 2 3))                 ;; bad

(letrec ((x (lambda (y)
	      (x 1 2 3))))   ;; bad
  (pk x))

(letrec () #f)
(letrec ())                  ;; bad

(letrec ()
  (define x 1)
  x)

(letrec ()                   ;; bad
  (define x 1))              ;; bad (unused)

(letrec ()
  (pk (noop))
  (define x 1)               ;; bad
  x)

(define letrec-test-proc1
  (letrec ((x 1)
	(y 2))
    (pk x)
    (pk y)
    (lambda (a b c) #f)))
(letrec-test-proc1 1)        ;; bad

(letrec ((foo (lambda ()     ;; bad (unused)
		(foo))))
  #f)


;;-----------------------------------------------------------------------------
;; list->string

(list->string)                  ;; bad
(pk (list->string '(#\a #\b)))
(list->string '(#\a #\b) 999)   ;; bad

(list->string '(99 #\b))        ;; bad
(list->string '(#\b 99))        ;; bad


;;-----------------------------------------------------------------------------
;; load

(load)                       ;; bad
(load 999)                   ;; bad
(load "/dev/null")
(load "/dev/null" 999)       ;; bad


;;-----------------------------------------------------------------------------
;; load-extension

(pk load-extension)
(load-extension)                      ;; bad
(load-extension "foo.so")             ;; bad
(load-extension "foo.so" "foo_init")  ;; bad, cannot load
(load-extension "foo.so" "foo_init" 'blah)  ;; bad

;; this gets considered "unused"
;; (load-extension "libguile.so" "scm_major_version")

(let ((load-extension-test1 (and (getenv "nosuchvar") "nosuchvar")))
  (load-extension "/lib/nosuchlibrary.so"   ;; bad, non-const
		  load-extension-test1))

(load-extension "/lib/nosuchlibrary.so"   ;; bad, non-const
		(noop nosuchdef))         ;; bad


;;-----------------------------------------------------------------------------
;; make-array

(pk (make-array #f (+ 1 2)))
(pk (make-array #f (list 1 2)))

(make-array #f -1)           ;; bad
(pk (make-array #f '(-5 -3)))
(make-array #f '(-3 -5))     ;; bad

(make-array #f "x")          ;; bad
(make-array #f 'x)           ;; bad

(make-array #f '(x 1))       ;; bad
(make-array #f '(1 x))       ;; bad
(make-array #f '("x" 1))     ;; bad
(make-array #f '(1 "x"))     ;; bad


;;-----------------------------------------------------------------------------
;; make-list

(pk make-list)
(make-list)                   ;; bad
(pk (make-list 5))
(pk (make-list 5 'x))
(make-list 5 'x 99)           ;; bad


;;-----------------------------------------------------------------------------
;; make-object-property

(pk make-object-property)
(pk (make-object-property))
(make-object-property 99)            ;; bad

(define make-object-property-test1 (make-object-property))
(make-object-property-test1)         ;; bad
(pk (make-object-property-test1 'x))
(make-object-property-test1 'x 99)   ;; bad


;;-----------------------------------------------------------------------------
;; make-procedure-with-setter

(make-procedure-with-setter (lambda () #f) (lambda () #f))          ;; bad
(pk (make-procedure-with-setter (lambda () #f) (lambda (val) #f)))

(define make-procedure-with-setter-test1
  (make-procedure-with-setter (lambda (x) #f) (lambda (x val) #f)))
(make-procedure-with-setter-test1 1 2 3)  ;; bad

(let ((proc (make-procedure-with-setter string-ref string-set!))
      (str  (make-string 3)))
  (proc "x" 1)
  (proc 'x 1)                 ;; bad
  (proc 'x 1 2)               ;; bad

  (set! (proc str 1) #\z)
  (set! (proc str 1 2) #\z))  ;; bad


;;-----------------------------------------------------------------------------
;; make-record-type

(pk make-record-type)
(make-record-type)                       ;; bad
(make-record-type "foo")                 ;; bad
(pk (make-record-type "foo" '(bar)))
(pk (make-record-type "foo" '(bar) noop))
(make-record-type "foo" '(bar) noop 99)  ;; bad


;;-----------------------------------------------------------------------------
;; make-shared-array

(define make-shared-array-test0 (make-array #f '(0 1) '(0 1)))

(pk make-shared-array)
(make-shared-array)                                        ;; bad
(make-shared-array make-shared-array-test0)                ;; bad
(make-shared-array make-shared-array-test0 list)           ;; bad
(pk (make-shared-array make-shared-array-test0 (lambda (x) (list x x)) '(0 1)))
(pk (make-shared-array make-shared-array-test0 list '(0 1) '(0 1)))

(make-shared-array make-shared-array-test0 (lambda () (list 0 0))    ;; bad
		   '(0 1))
(pk (make-shared-array make-shared-array-test0 (lambda (x) (list 0 0))
		   '(0 1)))
(make-shared-array make-shared-array-test0 (lambda (x y) (list 0 0)) ;; bad
		   '(0 1))


;;-----------------------------------------------------------------------------
;; make-stack

(pk make-stack)
(make-stack)        ;; bad
(make-stack 999)    ;; bad
(pk (make-stack #t))
(call-with-current-continuation (lambda (cont)
				  (make-stack cont)))

;;-----------------------------------------------------------------------------
;; %make-void-port

(pk %make-void-port)
(pk (%make-void-port 123))     ;; bad
(%make-void-port "w")          ;; bad (called for effect)
(pk (%make-void-port "w"))
(pk (%make-void-port "r"))
(pk (%make-void-port "XXX"))   ;; bad
(pk (%make-void-port "r" 123)) ;; bad


;;-----------------------------------------------------------------------------
;; map

(map (lambda () #f) '(123))            	 ;; bad
(map (lambda (x) #f) '(123))
(map (lambda (x y) #f) '(123))         	 ;; bad
(map (lambda (x y z) #f) '(123))       	 ;; bad

(map (lambda () #f) '(123) '(456))       ;; bad
(map (lambda (x) #f) '(123) '(456))      ;; bad
(map (lambda (x y) #f) '(123) '(456))
(map (lambda (x y z) #f) '(123) '(456))  ;; bad


;;-----------------------------------------------------------------------------
;; max

(pk max)
(max)                     ;; bad
(pk (max 1))
(pk (max 1 2))
(pk (max 1 2 3))

(max 1+i)                 ;; bad
(max 1+i 2-i)             ;; bad
(max 'x)                  ;; bad
(max "abc")               ;; bad


;;-----------------------------------------------------------------------------
;; member

(pk member)
(member)                  ;; bad
(member 1)                ;; bad
(pk (member 1 '(3 1 2)))
(member 1 '(3 1 2) 99)    ;; bad


;;-----------------------------------------------------------------------------
;; memq

(memq "x" '(1 2 3))       ;; bad


;;-----------------------------------------------------------------------------
;; merge

(pk merge)
(merge)                   ;; bad
(merge '())               ;; bad
(merge '() '())           ;; bad
(pk (merge '() '() <))
(merge '() '() abs)       ;; bad
(merge #f '() <)          ;; bad
(merge '() #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; merge!

(pk merge!)
(merge!)                  ;; bad
(merge! '())              ;; bad
(merge! '() '())          ;; bad
(merge! '() '() <)
(merge! '() '() abs)      ;; bad
(merge! #f '() <)         ;; bad
(merge! '() #f <)         ;; bad


;;-----------------------------------------------------------------------------
;; min

(pk min)
(min)                     ;; bad
(pk (min 1))
(pk (min 1 2))
(pk (min 1 2 3))

(min 1+i)                 ;; bad
(min 1+i 2-i)             ;; bad
(min 'x)                  ;; bad
(min "abc")               ;; bad


;;-----------------------------------------------------------------------------
;; object-properties

(pk object-properties)
(object-properties)                           ;; bad
(pk (object-properties 'foo))
(object-properties 'foo 'bar)                 ;; bad
(for-each display (object-properties 'foo))
(odd? (object-properties 'foo))               ;; bad


;;-----------------------------------------------------------------------------
;; object-property

(pk object-property)
(object-property)                             ;; bad
(object-property 'foo)                        ;; bad
(pk (object-property 'foo 'bar))
(object-property 'foo 'bar 999)               ;; bad


;;-----------------------------------------------------------------------------
;; open-file

(pk (open-file "name" "r"))
(pk (open-file "name" "w"))
(pk (open-file "name" "a"))
(open-file "name" "Z")    ;; bad
(open-file "name" "rZ")   ;; bad
(open-file "name" "rw")   ;; bad
(pk (open-file "name" "r+"))
(pk (open-file "name" "w0"))
(pk (open-file "name" "al"))
(open-file "name" "rl")   ;; bad
(open-file "name" "w0l")  ;; bad


;;-----------------------------------------------------------------------------
;; or

(pk or)
(or)                      ;; bad
(or 1)                    ;; bad
(or 1 2)
(or nodef #t)             ;; bad
(or #t nodef)
(or (not (defined? 'nodef))
    (nodef))

;;-----------------------------------------------------------------------------
;; port-for-each

(port-for-each)                            ;; bad
(port-for-each (lambda (x) #f))
(port-for-each (lambda (x) #f) '(123))     ;; bad

(port-for-each (lambda () #f))             ;; bad
(port-for-each (lambda (x) #f))
(port-for-each (lambda (x y) #f))          ;; bad
(port-for-each (lambda (x y z) #f))        ;; bad


;;-----------------------------------------------------------------------------
;; primitive-make-property

(pk primitive-make-property)
(primitive-make-property)                              ;; bad
(pk (primitive-make-property (lambda (prop obj) #f)))
(primitive-make-property (lambda (prop obj) #f) 123)   ;; bad

(primitive-make-property (lambda () #f))               ;; bad
(primitive-make-property (lambda (prop) #f))           ;; bad
(pk (primitive-make-property (lambda (prop obj) #f)))
(primitive-make-property (lambda (prop obj x) #f))     ;; bad


;;-----------------------------------------------------------------------------
;; primitive-property-del!

(pk primitive-property-del!)
(primitive-property-del!)                                        ;; bad
(primitive-property-del! (primitive-make-property #f))           ;; bad
(primitive-property-del! (primitive-make-property #f) 'foo)
(primitive-property-del! (primitive-make-property #f) 'foo 999)  ;; bad


;;-----------------------------------------------------------------------------
;; primitive-property-ref

(pk primitive-property-ref)
(primitive-property-ref)                                         ;; bad
(primitive-property-ref (primitive-make-property #f))            ;; bad
(pk (primitive-property-ref (primitive-make-property #f) 'foo))
(primitive-property-ref (primitive-make-property #f) 'foo 999)   ;; bad


;;-----------------------------------------------------------------------------
;; primitive-property-set!

(pk primitive-property-set!)
(primitive-property-set!)                                              ;; bad
(primitive-property-set! (primitive-make-property #f))                 ;; bad
(primitive-property-set! (primitive-make-property #f) 'foo)            ;; bad
(primitive-property-set! (primitive-make-property #f) 'foo 999)
(primitive-property-set! (primitive-make-property #f) 'foo 999 54321)  ;; bad


;;-----------------------------------------------------------------------------
;; procedure->macro

(pk procedure->macro)
(procedure->macro)                                 ;; bad
(pk (procedure->macro (lambda (expr env) #f)))
(procedure->macro (lambda (expr env) #f) 123)      ;; bad

(procedure->macro (lambda () #f))                  ;; bad
(procedure->macro (lambda (expr) #f))              ;; bad
(pk (procedure->macro (lambda (expr env) #f)))
(procedure->macro (lambda (expr env x) #f))        ;; bad

(define procedure->macro-test1
  (procedure->macro (lambda (expr env)
		      (cons 'car (cdr expr)))))
(procedure->macro-test1 '(x))
(procedure->macro-test1 '(x) '(y))   ;; bad

(define procedure->macro-test2 procedure->macro-test1)
(procedure->macro-test2 '(x))
(procedure->macro-test2 '(x) '(y))   ;; bad


;;-----------------------------------------------------------------------------
;; procedure->memoizing-macro

(pk procedure->memoizing-macro)
(procedure->memoizing-macro)                                 ;; bad
(pk (procedure->memoizing-macro (lambda (expr env) #f)))
(procedure->memoizing-macro (lambda (expr env) #f) 123)      ;; bad

(procedure->memoizing-macro (lambda () #f))                  ;; bad
(procedure->memoizing-macro (lambda (expr) #f))              ;; bad
(pk (procedure->memoizing-macro (lambda (expr env) #f)))
(procedure->memoizing-macro (lambda (expr env x) #f))        ;; bad

(define procedure->memoizing-macro-test1
  (procedure->memoizing-macro (lambda (expr env)
				(cons 'car (cdr expr)))))
(procedure->memoizing-macro-test1 '(x))
(procedure->memoizing-macro-test1 '(x) '(y))   ;; bad


;;-----------------------------------------------------------------------------
;; procedure->syntax

(pk procedure->syntax)
(procedure->syntax)                                 ;; bad
(pk (procedure->syntax (lambda (expr env) #f)))
(procedure->syntax (lambda (expr env) #f) 123)      ;; bad

(procedure->syntax (lambda () #f))                  ;; bad
(procedure->syntax (lambda (expr) #f))              ;; bad
(pk (procedure->syntax (lambda (expr env) #f)))
(procedure->syntax (lambda (expr env x) #f))        ;; bad


;;-----------------------------------------------------------------------------
;; quasiquote

`()
`(nodef)
`(,nodef)              ;; bad
`(((,nodef)))          ;; bad
`(((,noop . ,nodef)))  ;; bad

`(1 '(,nodef))         ;; bad
`(1 `(,nodef))
`(1 ,'nodef)

`(1 `(2 ,nodef))
`(1 `(2 ,(3 ,noop)))
`(1 `(2 ,(3 ,nodef)))  ;; bad

(quasiquote 1 2 (unquote 3))
(quasiquote 1 2 (unquote 3 4))  ;; bad

(let ((x 1))
  `(,x))

(let ((x 'nodef))
  `(list ',x))


;;-----------------------------------------------------------------------------
;; quit

(pk quit)
(quit)
(quit 0)
(quit 1 2) ;; bad


;;-----------------------------------------------------------------------------
;; quote

'x
'(x)
(quote x)

'#\x       ;; bad
'(#\x)

'"abc"     ;; bad
'("abc")

'123       ;; bad
'(123)
'1.5       ;; bad
'(1.5)

'#t        ;; bad
'(#t)
'#f        ;; bad
'(#f)

'#(1 2 3)


;;-----------------------------------------------------------------------------
;; raise

(pk raise)
(raise)                     ;; bad
(raise 123)
(raise "foo")               ;; bad
(raise "foo" "bar")         ;; bad


;;-----------------------------------------------------------------------------
;; random

(pk random)
(random)                                   ;; bad
(pk (random 123))
(pk (random 123 (seed->random-state 123)))
(random 123 (seed->random-state 123) 456)  ;; bad

(random "foo")                             ;; bad
(random 123 456)                           ;; bad


;;-----------------------------------------------------------------------------
;; random:exp

(pk random:exp)
(pk (random:exp))    
(pk (random:exp 123))                               ;; bad
(pk (random:exp (seed->random-state 123)))
(pk (random:exp (seed->random-state 123) 456))      ;; bad

(pk (random:exp "foo"))                             ;; bad
(pk (random:exp 123))                               ;; bad
(pk (random:exp 123))                               ;; bad


;;-----------------------------------------------------------------------------
;; random:normal

(pk random:normal)
(pk (random:normal))    
(pk (random:normal 123))                             ;; bad
(pk (random:normal (seed->random-state 123)))
(pk (random:normal (seed->random-state 123) 456))    ;; bad

(pk (random:normal "foo"))                           ;; bad
(pk (random:normal 123))                             ;; bad
(pk (random:normal 123))                             ;; bad


;;-----------------------------------------------------------------------------
;; random:uniform

(pk random:uniform)
(pk (random:uniform))    
(pk (random:uniform 123))                            ;; bad
(pk (random:uniform (seed->random-state 123)))
(pk (random:uniform (seed->random-state 123) 456))   ;; bad

(pk (random:uniform "foo"))                          ;; bad
(pk (random:uniform 123))                            ;; bad
(pk (random:uniform 123))                            ;; bad


;;-----------------------------------------------------------------------------
;; read-set!

(read-set! keywords #t)


;;-----------------------------------------------------------------------------
;; record-accessor

(pk record-accessor)
(let ((type (make-record-type "foo" '(bar))))
  (record-accessor)                             ;; bad
  (record-accessor type)                        ;; bad
  (pk (record-accessor type 'bar))
  (record-accessor type 'bar 99)                ;; bad

  (let ((rec ((record-constructor type) #f)))
    ((record-accessor type 'bar))               ;; bad
    ((record-accessor type 'bar) rec)
    ((record-accessor type 'bar) 99)            ;; notyet bad
    ((record-accessor type 'bar) rec 99)))      ;; bad


;;-----------------------------------------------------------------------------
;; record-constructor

(pk record-constructor)
(let ((type (make-record-type "foo" '(bar))))
  (record-constructor)                          ;; bad
  (pk (record-constructor type))
  (record-constructor type 'bar)                ;; bad
  (pk (record-constructor type '(bar)))
  (record-constructor type '(bar) 99)           ;; bad

  (pk (record-constructor type '()))
  (record-constructor type '(123)))             ;; bad

(let ((type (make-record-type "foo" '(bar quux xyzzy))))
  (record-constructor type '(quux xyzzy)))


;;-----------------------------------------------------------------------------
;; record-modifier

(pk record-modifier)
(let ((type (make-record-type "foo" '(bar))))
  (record-modifier)                             ;; bad
  (record-modifier type)                        ;; bad
  (pk (record-modifier type 'bar))
  (record-modifier type 'bar 99)                ;; bad

  (let ((rec ((record-constructor type) #f)))
    ((record-modifier type 'bar))               ;; bad
    ((record-modifier type 'bar) rec)           ;; bad
    ((record-modifier type 'bar) rec 99)
    ((record-modifier type 'bar) 99 99)         ;; notyet bad
    ((record-modifier type 'bar) rec 123 456))) ;; bad


;;-----------------------------------------------------------------------------
;; record-predicate

(pk record-predicate)
(let ((type (make-record-type "foo" '(bar))))
  (record-predicate)                             ;; bad
  (pk (record-predicate type))
  (record-predicate type 'bar)                   ;; bad

  (let ((rec ((record-constructor type) #f)))
    ((record-predicate type))               ;; bad
    (pk ((record-predicate type) 'x))
    (pk ((record-predicate type) rec))
    ((record-predicate type) rec 99)))      ;; bad


;;-----------------------------------------------------------------------------
;; restricted-vector-sort!

(pk restricted-vector-sort!)
(restricted-vector-sort!)                   ;; bad
(restricted-vector-sort! #(1 2))            ;; bad
(restricted-vector-sort! #(1 2) <)          ;; bad
(restricted-vector-sort! #(1 2) < 0)        ;; bad
(restricted-vector-sort! #(1 2) < 1 2)
(restricted-vector-sort! #(1 2) abs 1 2)    ;; bad
(restricted-vector-sort! #f < 1 2)          ;; bad


;;-----------------------------------------------------------------------------
;; scm-error

(pk scm-error)
(scm-error)                                     	 ;; bad
(scm-error 'misc-error)                         	 ;; bad
(scm-error 'misc-error "foo")                   	 ;; bad
(scm-error 'misc-error "foo" "msg")             	 ;; bad
(scm-error 'misc-error "foo" "msg" '())                  ;; bad
(scm-error 'misc-error "foo" "msg" '() 'data)            ;; bad
(scm-error 'misc-error "foo" "msg" '() '(data))
(scm-error 'misc-error "foo" "msg" '() #f)
(scm-error 'misc-error "foo" "msg" '() '() 99)  	 ;; bad

(scm-error 'misc-error "foo" "~" '(123) '())    	 ;; bad
(scm-error 'misc-error "foo" "~~" '() '())
(scm-error 'misc-error "foo" "~a" '(123) '())
(scm-error 'misc-error "foo" "~d" '(123) '())   	 ;; bad
(scm-error 'misc-error "foo" "~~~d" '(123) '())          ;; bad
(scm-error 'misc-error "foo" "~~~a" '(123) '())

(scm-error 'misc-error "foo" "zz ~" '(123) '())          ;; bad
(scm-error 'misc-error "foo" "zz ~~" '() '())
(scm-error 'misc-error "foo" "zz ~a" '(123) '())
(scm-error 'misc-error "foo" "zz ~d" '(123) '())         ;; bad
(scm-error 'misc-error "foo" "zz ~~~d" '(123) '())       ;; bad
(scm-error 'misc-error "foo" "zz ~a~d" '(123 456) '())   ;; bad
(scm-error 'misc-error "foo" "zz ~~~a" '(123) '())
(scm-error 'misc-error "foo" "zz ~~ yy ~a" '(123) '())
(scm-error 'misc-error "foo" "zz ~~ yy ~a xx" '(123) '())


;;-----------------------------------------------------------------------------
;; seek

(pk seek)
(seek)                 ;; bad
(seek 0)               ;; bad
(seek 0 0)             ;; bad
(seek 0 0 SEEK_CUR)
(seek 0 0 SEEK_CUR 0)  ;; bad

(seek 0 0 SEEK_CUR)
(seek 0 0 SEEK_SET)
(seek 0 0 SEEK_END)
(seek 0 0 999)         ;; bad

(seek 'x 0 SEEK_CUR)   ;; bad
(seek 0 'x SEEK_CUR)   ;; bad
(seek 0 0 'x)          ;; bad

(seek "x" 0 SEEK_CUR)  ;; bad
(seek 0 "x" SEEK_CUR)  ;; bad
(seek 0 0 "x")         ;; bad

(seek (current-input-port) 0 SEEK_END)
(seek (current-output-port) 0 SEEK_END)


;;-----------------------------------------------------------------------------
;; set!

(pk set!)
(set!)                                     ;; bad
(set! %load-hook)                          ;; bad
(set! %load-hook (lambda (filename) #f))
(set! 123 456)                             ;; bad
(set! set!-nodef 456)                      ;; bad

(define set!-test1 #f)
(set! set!-test1 (lambda (x) #f))
(set!-test1 1 2 3)                         ;; bad

(define (set!-test2 x) #f)
(set! set!-test2
      (set!-test2 1 2 3))                  ;; bad

(define set!-test3 #f)
(set! set!-test3 #f)
(set! set!-test3 #f)
(set! set!-test3 #f)

(define set!-test4 (lambda (x) #f))
(set! set!-test4 (lambda (x y) #f))
(set!-test4 1)                             ;; bad
(set!-test4 1 2)


;;-----------------------------------------------------------------------------
;; set-object-properties!

(pk set-object-properties!)
(set-object-properties!)                      ;; bad
(set-object-properties! 'foo)                 ;; bad
(set-object-properties! 'foo 999)             ;; bad
(set-object-properties! 'foo '())
(set-object-properties! 'foo '((a . 999)))
(odd? (set-object-properties! 'foo '()))      ;; bad


;;-----------------------------------------------------------------------------
;; set-object-property!

(pk set-object-property!)
(set-object-property!)                        ;; bad
(set-object-property! 'foo)                   ;; bad
(set-object-property! 'foo 'bar)              ;; bad
(set-object-property! 'foo 'bar 999)
(set-object-property! 'foo 'bar 999 123)      ;; bad
(odd? (set-object-property! 'foo 'bar 999))   ;; bad


;;-----------------------------------------------------------------------------
;; sethostent

(pk sethostent)
(sethostent)
(sethostent #t)
(sethostent #t 999) ;; bad


;;-----------------------------------------------------------------------------
;; setlocale

(pk (setlocale LC_ALL))
(setlocale LC_ALL "C")
(setlocale LC_MESSAGES "C")
(setlocale)                 ;; bad
(setlocale LC_ALL)          ;; not yet bad, return value unused
(setlocale LC_ALL 12345)    ;; bad
(setlocale -123 "C")        ;; bad
(setlocale LC_ALL "C" 999)  ;; bad


;;-----------------------------------------------------------------------------
;; setnetent

(pk setnetent)
(setnetent)
(setnetent #t)
(setnetent #t 999) ;; bad


;;-----------------------------------------------------------------------------
;; setprotoent

(pk setprotoent)
(setprotoent)
(setprotoent #t)
(setprotoent #t 999) ;; bad


;;-----------------------------------------------------------------------------
;; setservent

(pk setservent)
(setservent)
(setservent #t)
(setservent #t 999) ;; bad


;;-----------------------------------------------------------------------------
;; simple-format

(format "number ~a" 123)         ;; bad

(simple-format #t "" 1)          ;; bad
(simple-format #t "~a" 1)
(simple-format #t "~a ~a" 1 2)
(simple-format #t "~a~a~a" 1 2 3)
(simple-format #t "~s~s~s" 1 2 3)
(simple-format #t "~")     	 ;; bad
(simple-format #t "~a")    	 ;; bad
(simple-format #t "~x" 1)  	 ;; bad
(simple-format #t "~%~a" 1)

(let ((args '("x" 1 2 3)))
  (apply format #f "blah blah ~a~2,'0d~2,'0d.txt" args)) ;; bad


;;-----------------------------------------------------------------------------
;; sort

(pk sort)
(sort)               ;; bad
(sort '())           ;; bad
(pk (sort '() <))
(pk (sort #(1 2) <))
(sort '() abs)       ;; bad
(sort #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; sort!

(pk sort!)
(sort!)               ;; bad
(sort! '())           ;; bad
(sort! '() <)
(sort! #(1 2) <)
(sort! '() abs)       ;; bad
(sort! #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; sort-list

(pk sort-list)
(sort-list)               ;; bad
(sort-list '())           ;; bad
(pk (sort-list '() <))
(sort-list #(1 2) <)      ;; bad
(sort-list '() abs)       ;; bad
(sort-list #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; sorted?

(pk sorted?)
(sorted?)               ;; bad
(sorted? '())           ;; bad
(pk (sorted? '() <))
(pk (sorted? #(1 2) <))
(sorted? '() abs)       ;; bad
(sorted? #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; stable-sort

(pk stable-sort)
(stable-sort)               ;; bad
(stable-sort '())           ;; bad
(pk (stable-sort '() <))
(pk (stable-sort #(1 2) <))
(stable-sort '() abs)       ;; bad
(stable-sort #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; stable-sort!

(pk stable-sort!)
(stable-sort!)               ;; bad
(stable-sort! '())           ;; bad
(stable-sort! '() <)
(stable-sort! #(1 2) <)
(stable-sort! '() abs)       ;; bad
(stable-sort! #f <)          ;; bad


;;-----------------------------------------------------------------------------
;; stack-id

(pk stack-id)
(stack-id)                                   ;; bad
(stack-id 'xyz)                              ;; bad
(pk (stack-id #t))
(call-with-current-continuation stack-id)
(call-with-current-continuation
 (lambda (cont)
   (stack-id cont)))
(pk (stack-id (start-stack 'id 1)))


;;-----------------------------------------------------------------------------
;; start-stack

(start-stack 'id)        ;; bad
(start-stack 'id 1)
(start-stack 'id 1 2)    ;; bad

(let ((x 1))
  (start-stack 'id (begin (pk x) x)))

(start-stack 'id nodef)  ;; bad


;;-----------------------------------------------------------------------------
;; string

(pk (string))
(pk (string #\x))
(pk (string 123))                  ;; bad
(pk (string #\x #\y))
(pk (apply string '(#\x #\y)))     ;; bad, can use `list->string'


;;-----------------------------------------------------------------------------
;; string-append

(pk string-append)
(pk (string-append))                         ;; bad
(pk (string-append "x"))                     ;; bad (warn)
(pk (string-append "x" "y"))
(pk (string-append "x" 'zzz))                ;; bad


;;-----------------------------------------------------------------------------
;; substring-fill!

(pk substring-fill!)
(substring-fill!)                                 ;; bad
(substring-fill! (make-string 6 #\x))             ;; bad
(substring-fill! (make-string 6 #\x) 0)           ;; bad
(substring-fill! (make-string 6 #\x) 0 3)         ;; bad
(substring-fill! (make-string 6 #\x) 0 3 #\y)
(substring-fill! (make-string 6 #\x) 0 3 #\y 9)   ;; bad
(substring-fill! 'x 0 3 #\y)                      ;; bad
(substring-fill! (make-string 6 #\x) 'x 3 #\y)    ;; bad
(substring-fill! (make-string 6 #\x) 0 'x #\y)    ;; bad
(substring-fill! (make-string 6 #\x) 0 3 'x)      ;; bad


;;-----------------------------------------------------------------------------
;; t -- elisp removed from guile-lint checks

t                         ;; bad


;;-----------------------------------------------------------------------------
;; vector

(pk (vector))
(pk (vector 1))
(pk (vector 1 2))
(pk (apply vector '(1 2)))     ;; bad, can use `list->vector'


;;-----------------------------------------------------------------------------
;; waitpid

(waitpid -1)
(waitpid 0)
(waitpid 0 0)
(waitpid 0 WNOHANG)
(waitpid 0 WUNTRACED)
(waitpid 0 (logior WNOHANG WUNTRACED))
(waitpid)         ;; bad
(waitpid 0 -123)  ;; bad
(waitpid 0 0 0)   ;; bad


;;-----------------------------------------------------------------------------
;; while

(while #f)
(while #f (pk #t))
(while #t (pk #f))

(while nodef #f)          ;; bad
(while #t nodef)          ;; bad


;;-----------------------------------------------------------------------------
;; with-fluids*

(pk with-fluids*)
(with-fluids*)                             ;; bad
(with-fluids* '())                         ;; bad
(with-fluids* '() '())                     ;; bad
(with-fluids* '() '() (lambda () #f))
(with-fluids* '() '() (lambda () #f) 123)  ;; bad

(with-fluids* '() '() (lambda (x) #f))     ;; bad


;;-----------------------------------------------------------------------------
;; with-fluids

(define with-fluids-var (make-fluid))

(with-fluids)                          ;; bad
(with-fluids ((with-fluids-var 1))     ;; bad
  )
(with-fluids ((with-fluids-var 1))
  #f)


;;-----------------------------------------------------------------------------
;; with-input-from-file

(pk with-input-from-file)
(with-input-from-file)                                   ;; bad
(with-input-from-file "/etc/passwd")                     ;; bad
(with-input-from-file "/etc/passwd" (lambda () #f))
(with-input-from-file "/etc/passwd" (lambda () #f) 123)  ;; bad

(with-input-from-file "/etc/passwd" (lambda () #f))
(with-input-from-file "/etc/passwd" (lambda (x) #f))     ;; bad


;;-----------------------------------------------------------------------------
;; with-output-to-file

(pk with-output-to-file)
(with-output-to-file)                                 ;; bad
(with-output-to-file "/tmp/foo")                      ;; bad
(with-output-to-file "/tmp/foo" (lambda () #f))
(with-output-to-file "/tmp/foo" (lambda () #f) 123)   ;; bad

(with-output-to-file "/tmp/foo" (lambda () #f))
(with-output-to-file "/tmp/foo" (lambda (x) #f))      ;; bad


;;-----------------------------------------------------------------------------
;; with-error-to-file

(pk with-error-to-file)
(with-error-to-file)                                   ;; bad
(with-error-to-file "/tmp/foo")                     ;; bad
(with-error-to-file "/tmp/foo" (lambda () #f))
(with-error-to-file "/tmp/foo" (lambda () #f) 123) ;; bad

(with-error-to-file "/tmp/foo" (lambda () #f))
(with-error-to-file "/tmp/foo" (lambda (x) #f))     ;; bad


;;-----------------------------------------------------------------------------
;; undefine

(define ud 1)
(undefine 'ud)            ;; bad

(undefine ud)
(undefine ud)             ;; bad

(undefine 123)            ;; bad
(undefine x y z)          ;; bad

(let ()
  (undefine x))           ;; bad


;;-----------------------------------------------------------------------------
;; use-modules

(pk use-modules)
(use-modules)
(use-modules (nosuch mod))      ;; bad

(let ()
  (use-modules (ice-9 optargs)) ;; bad (not toplevel)
  lambda*)


;;-----------------------------------------------------------------------------
;; use-syntax

(pk use-syntax)
(use-syntax)
(use-syntax (nosuch mod))      ;; bad
(use-syntax (ice-9 streams))
(pk stream-map)

(let ()
  (use-syntax (ice-9 common-list)) ;; bad (not toplevel)
  and?)


;;-----------------------------------------------------------------------------
;; vectors

(pk #(1 2 3))
(pk '#(1 2 3))


;;-----------------------------------------------------------------------------
;; version

(pk version)
(pk (version))
(version 123)                  ;; bad
(pk (string-append (version) "x"))
(append (version) '(1 2))      ;; bad


;;-----------------------------------------------------------------------------
;; hint simple-format

(define (hint-simple-format0 fmt . args)
  #f)
''(guile-lint simple-format hint-simple-format0)
''(guile-lint simple-format hint-simple-format0)  ;; bad
(hint-simple-format0 "")
(hint-simple-format0 "~a")       ;; bad
(hint-simple-format0 "~a" 1)
(hint-simple-format0 "~a" 1 2)   ;; bad

(define (hint-simple-format00 fmt . args)
  #f)
''(guile-lint simple-format hint-simple-format00 0)
(hint-simple-format00 "")
(hint-simple-format00 "~a")       ;; bad
(hint-simple-format00 "~a" 1)
(hint-simple-format00 "~a" 1 2)   ;; bad

(define (hint-simple-format1 port fmt . args)
  #f)
''(guile-lint simple-format hint-simple-format1 1)
(hint-simple-format1 #f "")
(hint-simple-format1 #f "~a")       ;; bad
(hint-simple-format1 #f "~a" 1)
(hint-simple-format1 #f "~a" 1 2)   ;; bad


;;-----------------------------------------------------------------------------
;; hint ice-9-format

(define (hint-ice-9-format0 fmt . args)
  #f)
''(guile-lint ice-9-format hint-ice-9-format0)
''(guile-lint ice-9-format hint-ice-9-format0)  ;; bad
(hint-ice-9-format0 "")
(hint-ice-9-format0 "~a")        ;; bad
(hint-ice-9-format0 "~a" 1)
(hint-ice-9-format0 "~a" 1 2)    ;; bad

(define (hint-ice-9-format00 fmt . args)
  #f)
''(guile-lint ice-9-format hint-ice-9-format00 0)
(hint-ice-9-format00 "")
(hint-ice-9-format00 "~a")       ;; bad
(hint-ice-9-format00 "~a" 1)
(hint-ice-9-format00 "~a" 1 2)   ;; bad

(define (hint-ice-9-format1 port fmt . args)
  #f)
''(guile-lint ice-9-format hint-ice-9-format1 1)
(hint-ice-9-format1 #f "")
(hint-ice-9-format1 #f "~a")     ;; bad
(hint-ice-9-format1 #f "~a" 1)
(hint-ice-9-format1 #f "~a" 1 2) ;; bad


;;-----------------------------------------------------------------------------
;; (ice-9 r5rs) null-environment

(use-modules (ice-9 r5rs))
(pk null-environment)
(null-environment)      ;; bad
(pk (null-environment 5))
(null-environment 1 2)  ;; bad


;;-----------------------------------------------------------------------------
;; misc

)     ;; bad

;; check tail recursion in stray ) error catch
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) ;; bad
