;; Test module (ice-9 format).

;; Copyright 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(use-modules (ice-9 format))


;;-----------------------------------------------------------------------------
;; misc

;; not enough args
(format "abc")                             	  ;; bad

;; missing port arg
(format "abc ~a" 1)                        	  ;; bad

(format #f "abc ~a" 1)
(format #f "blah ~a" 1 2)                         ;; bad
(format #f "~Z")                                  ;; bad


;;-----------------------------------------------------------------------------
;; arguments used

(format #f "~a ~1*~a" 1 2 3)   	   ;; bad
(format #f "~a ~1*~a" 1 2)     	   ;; bad
(format #f "~a ~2*~a" 1 2)     	   ;; bad


;;-----------------------------------------------------------------------------
;; parameter separators

(format #f "~2,'0d" 123)
(format #f "~2'0d" 123)       ;; bad

(format #f "~2,#a" 123)
(format #f "~2#a" 123)        ;; bad

(format #f "~2,va" 1 123)
(format #f "~2va" 1 123)      ;; bad

(format #f "~6,'x,'y:d" 123)
(format #f "~6,'x'y:d" 123)   ;; bad

(format #f "~,,'x,2:d" 123)
(format #f "~,,'x2:d" 123)    ;; bad

(format #f "~,,'x,v:d" 123 1)
(format #f "~,,'xv:d" 123 1)  ;; bad

(format #f "~,,'x,#:d" 123)
(format #f "~,,'x#:d" 123)    ;; bad

(format #f "~#,#a" 123)
(format #f "~##a" 123)        ;; bad

(format #f "~#,2a" 123)
(format #f "~#2a" 123)        ;; bad

(format #f "~#,va" 123 1)
(format #f "~#va" 123 1)      ;; bad

(format #f "~#,'xd ~d~d~d" 123 9 9 9)
(format #f "~#'xd ~d~d~d" 123 9 9 9)  ;; bad


;;-----------------------------------------------------------------------------
;; number parameter

(format #f "~2d" 123)
(format #f "~+2d" 123)
(format #f "~,,-2f" 123)
(format #f "~+d" 123)        ;; bad
(format #f "~,,-f" 123)      ;; bad
(format #f "~,,-+2f" 123)    ;; bad
(format #f "~,,+-2f" 123)    ;; bad


;;-----------------------------------------------------------------------------
;; ~~  tilde

(format #f "~")                ;; bad
(format #f "~~")
(format #f "~~~")              ;; bad

(format #f "~6~")

(format #f "~@~")              ;; bad
(format #f "~:~")              ;; bad
(format #f "~@:~")             ;; bad


;;-----------------------------------------------------------------------------
;; ~!  flush

(format #f "~!")
(format #f "~6!")              ;; bad
(format #f "~:!")              ;; bad
(format #f "~@!")              ;; bad


;;-----------------------------------------------------------------------------
;; ~*  move

(format #f "~'x*")             ;; bad


;;-----------------------------------------------------------------------------
;; ~?  subformat

(format #f "~?" "some-fmt" '(1 2))
(format #f "~:?" "some-fmt" '(1 2))           ;; bad
(format #f "~*~?" 'unused "some-fmt" '(1 2))  ;; bad
(format #f "~@?"           "some-fmt" 1 2)
(format #f "~*~@?" 'unused "some-fmt" 1 2)    ;; bad
(format #f "~2*~@?" 'unused1 'unused2 "some-fmt" 1 2)    ;; bad


;;-----------------------------------------------------------------------------
;; ~( ~)  case

(format #f "~(blah~)")
(format #f "~@(blah~)")
(format #f "~:(blah~)")
(format #f "~@:(blah~)")
(format #f "~@:(blah~)")

(format #f "~(")               ;; bad
(format #f "~)")               ;; bad
(format #f "~6(")              ;; bad
(format #f "blah~)")           ;; bad

(format #f "~( ~( ~) ~)")      ;; bad


;;-----------------------------------------------------------------------------
;; ~{ ~}  iteration

(format #f "~1,2{")            ;; bad
(format #f "~}")               ;; bad
(format #f "~{")               ;; bad
(format #f "~{" '(1 2))        ;; bad

(format #f "~{~d~}")           ;; bad
(format #f "~{~d~}" '(1 2 3))
(format #f "~@{~d~}" 1 2 3)
(format #f "~:@{~d~}" '(1) '(2) '(3))
(format #f "~{~{~d~}~}" '((1 2 3)))
(format #f "~{~{~d~}" '((1 2 3)))       ;; bad
(format #f "~{~d~}~}" '((1 2 3)))       ;; bad

(format #f "~{~d~} ~s" '(1 2) "foo")
(format #f "~{~d~} ~s" '(1 2))          ;; bad
(format #f "~{~#d~} ~s" '(1 2) "foo")
(format #f "~{~#d~} ~s" '(1 2))         ;; bad
(format #f "~{~#*~} ~s" '(1 2) "foo")
(format #f "~{~#*~} ~s" '(1 2))         ;; bad

(format #f "~{~} ~s" "~d" '(1 2) "foo")
(format #f "~{~} ~s" "~d" '(1 2))       ;; bad

(format #f "~{~0[x~]~}" '(1 2))
(format #f "~{~0[x~}~}" '(1 2))         ;; bad


;;-----------------------------------------------------------------------------
;; ~[ ~; ~]  conditional

(format #f "~[")               	;; bad
(format #f "~[~]")             	;; bad
(format #f "~[~]" 0)
(format #f "~0[~]")
(format #f "~0[~;~]")
(format #f "~;~]")             	;; bad
(format #f "~0[~0[~]~]")
(format #f "~0[~0[~]")         	;; bad

(format #f "~:[x~]" 0)         	;; bad
(format #f "~:[x~;y~]" 0)
(format #f "~:[x~;y~;z~]" 0)   	;; bad

(format #f "~@[x~]" 0)
(format #f "~@[x~;y~]" 0)      	;; bad
(format #f "~@[x~;y~;z~]" 0)   	;; bad

(format #f "~@:[x~]" 0)        	;; bad

(format #f "~;")               	;; bad

(format #f "~0[~{x~}~]" '(1 2))
(format #f "~0[~{x~]~}" '(1 2)) ;; bad

(format #f "~0[a~]")
(format #f "~0[a~]" 99)         ;; bad
 

;;-----------------------------------------------------------------------------
;; ~^ escape

(format #f "~(abc~^def~)")
(format #f "~( ~d ~^ ~d ~)" 1)      ;; bad
(format #f "~( ~d ~^ ~d ~)" 1 2)
(format #f "~( ~d ~^ ~d ~)" 1 2 3)  ;; bad

(format #f "~{ ~d ~^ ~d ~}" '(1 2 3))

(format #f "~[ ~d ~^ ~; ~a ~]" 0)


;;-----------------------------------------------------------------------------
;; ~a  display

(format #f "~a")               ;; bad
(format #f "~a" 1)
(format #f "~a" 1 2)           ;; bad
(format #f "~+a" 1)            ;; bad
(format #f "~-a" 1)            ;; bad
(format #f "~-0a" 1)
(format #f "~+0a" 1)

(format #f "~a" 1 2)           ;; bad
(format #f "~a ~a ~a" 1 2)     ;; bad
(format #f "~a ~:p" 1)
(format #f "~a ~p" 1)          ;; bad

(format #f "~:a" 1)
(format #f "~::a" 1)           ;; bad
(format #f "~@a" 1)
(format #f "~@@a" 1)           ;; bad

(format #f "~#a" 1)
(format #f "~#*~a" 1)          ;; bad
(format #f "~#*~1:*~a" 1)

(format #f "~6,,,'Xa" 123)
;; padchar without minwidth
(format #f "~,,,'Xa" 123)       ;; bad


;;-----------------------------------------------------------------------------
;; ~c  character

(format #f "~c" #\z)
(format #f "~:c" #\z)
(format #f "~@c" #\z)


;;-----------------------------------------------------------------------------
;; ~d  decimal

(format #f "~d" 123)

;; minwidth
(format #f "~6d" 123)
(format #f "~2d" 123)
(format #f "~1d" 123)          ;; bad
(format #f "~0d" 123)          ;; bad
(format #f "~-1d" 123)         ;; bad
(format #f "~-6d" 123)         ;; bad

;; padchar
(format #f "~6,'*d" 123)
(format #f "~6,2d" 123)        ;; bad

;; commachar
(format #f "~,,'-:d" 123)
(format #f "~,,2:d" 123)       ;; bad

;; commawidth
(format #f "~,,,2:d" 123)
(format #f "~,,,-1:d" 123)     ;; bad
(format #f "~,,,'x:d" 123)     ;; bad

;; too many
(format #f "~,,,,99d" 123)     ;; bad

;; padchar without minwidth
(format #f "~,'*d" 123)        ;; bad
;; commachar without :
(format #f "~,,'Xd" 123)       ;; bad
;; commawidth without :
(format #f "~,,,2d" 123)       ;; bad


;;-----------------------------------------------------------------------------
;; ~f  fixed point

(format #f "~f" 123)

;; padchar without width
(format #f "~8,,,,'*f" 123)
(format #f "~,,,,'*f" 123)     ;; bad

;; overflowchar without width
(format #f "~8,,,'*f" 123)
(format #f "~,,,'*f" 123)      ;; bad


;;-----------------------------------------------------------------------------
;; ~q  inquiry

(format #f "~q")
(format #f "~:q")
(format #f "~@q")              ;; bad
(format #f "~@:q")             ;; bad


;;-----------------------------------------------------------------------------
;; ~r  radix, roman, words

(format #f "~2r" 1)
(format #f "~1r" 1)            ;; bad


(format #f "~|~%~&~_~/~~")
(format #f "~@|")              ;; bad
(format #f "~:|")              ;; bad
(format #f "~@:|")             ;; bad
(format #f "~@%")              ;; bad
(format #f "~:%")              ;; bad
(format #f "~@:%")             ;; bad
(format #f "~@&")              ;; bad
(format #f "~:&")              ;; bad
(format #f "~@:&")             ;; bad
(format #f "~@_")              ;; bad
(format #f "~:_")              ;; bad
(format #f "~@:_")             ;; bad
(format #f "~@/")              ;; bad
(format #f "~:/")              ;; bad
(format #f "~@:/")             ;; bad

(format #f "~8|")

;; minwidth
(format #f "~3,6r" 123)
(format #f "~3,2r" 123)
(format #f "~3,1r" 123)        ;; bad
(format #f "~3,0r" 123)        ;; bad
(format #f "~3,-1r" 123)       ;; bad
(format #f "~3,-6r" 123)       ;; bad

;; padchar without minwidth
(format #f "~3,,'*d" 123)      ;; bad
;; commachar without :
(format #f "~3,,'Xd" 123)      ;; bad
;; commawidth without :
(format #f "~3,,,2d" 123)      ;; bad
