;; Test module (ice-9 optargs).

;; Copyright 2003, 2004, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

define*          ;; bad
define*-public   ;; bad
lambda*          ;; bad

(use-modules (ice-9 optargs))


;;-----------------------------------------------------------------------------
;; define*

(define* define*-test1 (lambda (x) #f))  ;; bad
(define*-test1)                          ;; bad
(define*-test1 1)
(define*-test1 1 2)                      ;; bad
(define*-test1 1 2 3)                    ;; bad

(define* (define*-test2 x #:optional y) #f)
(define*-test2)                          ;; bad
(define*-test2 1)
(define*-test2 1 2)
(define*-test2 1 2 3)                    ;; bad

(define* (define*-test3 lst #:optional (l= equal?))
  l=)
(pk define*-test3)

(define* (define*-test4 #:optional x)
  (define bar 1)
  bar)
(pk define*-test4)

(define* (define*-test5 #:optional define*-test5-arg)
  #f)
(pk define*-test5)
define*-test5-arg     ;; bad

(define* define*-test6)                  ;; bad
(define* define*-test7 123)              ;; bad

(define* ((define*-test8 #:optional bar) #:optional baz) #f)
(pk define*-test8)

(define* ((define*-test9 #:optional bar)
	  #:optional
	  (baz nodef))                   ;; bad
  #f)
(pk define*-test9)

(define* (define*-test10 x y)   #f)      ;; bad, can be define
(pk define*-test10)

(define* (define*-test11 x . y) #f)      ;; bad, can be define
(pk define*-test11)

(define* (define*-test12 #:key x) #f)
(define*-test12 #:x 123)
(define*-test12 #:x)                 ;; bad
(define*-test12 #:x 123 #:y)         ;; bad
(define*-test12 #:x 123 #:y 123)     ;; bad
(define*-test12 #:x 123 1 2 3)       ;; bad

(define* (define*-test13 #:key x #:allow-other-keys) #f)
(define*-test13 #:x 123)
(define*-test13 #:x)                 ;; bad
(define*-test13 #:x 123 #:y)         ;; bad
(define*-test13 #:x 123 #:y 123)
(define*-test13 #:x 123 1 2 3)       ;; bad

(define* (define*-test14 #:key x #:rest r) #f)
(define*-test14 #:x 123)
(define*-test14 #:x)                 ;; bad
(define*-test14 #:x 123 #:y)         ;; bad
(define*-test14 #:x 123 #:y 123)     ;; bad
(define*-test14 #:x 123 1 2 3)

(define* (define*-test15 #:key x . r) #f)
(define*-test15 #:x 123)
(define*-test15 #:x)                 ;; bad
(define*-test15 #:x 123 #:y)         ;; bad
(define*-test15 #:x 123 #:y 123)     ;; bad
(define*-test15 #:x 123 1 2 3)

(define* (define*-test16 #:optional (x 1) (y 2)) #f)
(define*-test16)
(define*-test16 1)
(define*-test16 1 2)
(define*-test16 1 2 3)               ;; bad
(define*-test16 1 2 3 4)             ;; bad

(define* (define*-test17 #:optional (x 1) (y 2) #:rest r) #f)
(define*-test17)
(define*-test17 1)
(define*-test17 1 2)
(define*-test17 1 2 3)
(define*-test17 1 2 3 4)

(define* (define*-test17 x #:optional (y 1) z) #f)
(define*-test17)                     ;; bad
(define*-test17 1)
(define*-test17 1 2)
(define*-test17 1 2 3)
(define*-test17 1 2 3 4)             ;; bad

(define* (define*-test18 #:key
	   (name   "<no name>")
	   (reduce #f)
	   (min    noop)
	   (max    noop))
  #f)
(define*-test18 #:name "boll")
(define*-test18 #:name "boll" #:blah 123) ;; bad
(define*-test18 #:name "boll" #:min)      ;; bad

(define* (define*-test19 #:key (name "<no name>") (min noop)
	   #:allow-other-keys)
  #f)
(define*-test19 #:name "boll")
(define*-test19 #:name "boll" #:blah 123)
(define*-test19 #:name "boll" #:min)      ;; bad

(define* (define*-test20 x #:optional (y x))
  #f)
(pk define*-test20)

(define* (define*-test21 #:optional foo)
  "docstring"
  (define (x) #f)
  (x))
(pk define*-test21)

(define* (define*-test22 #:optional foo)
  "docstring"
  (define (x) 123)
  (define (y) 456)
  (+ (x) (y)))
(pk define*-test22)

(define* (define*-test23 #:optional foo)
  "docstring"
  (define (x) 123)
  (pk "another string")
  (define (y) 456)                        ;; bad
  (+ (x) (y)))
(pk define*-test23)

(define* (define*-test24 #:optional y)    ;; bad (never used)
  #f)

(define* (or-map #:optional x) x)         ;; bad (shadow import)
(pk or-map)

(define* and-map #f)                      ;; bad (shadow and can be define)
(pk and-map)

(define* (define*-test25 #:key x (y (error "must have y")))
  (pk x y))
(define*-test25 #:x 123 #:y 456)
(define*-test25 #:y 456)
(define*-test25 #:x 123)                  ;; bad

(define* (define*-test26 #:optional (bar '()))
  (make-list 5 bar))
(pk define*-test26)


;;-----------------------------------------------------------------------------
;; define*-public

(define*-public (define*-public-test1 #:optional x)
  #f)

(define*-public (define*-public-test2 #:optional x)
  (define bar 1)
  bar)

(define*-public ((define*-public-test8 #:optional bar) #:optional baz) #f)
(define*-public ((define*-public-test9 #:optional bar)
		 #:optional
		 (baz nodef))            ;; bad
  #f)

(define*-public (define*-public-test21 #:optional foo)
  "docstring"
  (define (x) #f)
  (x))

(define*-public (define*-public-test22 #:optional foo)
  "docstring"
  (define (x) 123)
  (define (y) 456)
  (+ (x) (y)))

(define*-public (define*-public-test23 #:optional foo)
  "docstring"
  (define (x) 123)
  (pk "another string")
  (define (y) 456)                       ;; bad
  (+ (x) (y)))


;;-----------------------------------------------------------------------------
;; lambda*

(lambda* (#:optional x) #f)
(lambda* (a #:optional a) #f)  ;; bad
(lambda* (a #:key a) #f)       ;; bad

(lambda* (a #:blahblah)        ;; bad
  #f)

(lambda* (#:optional xyz)
  (define x 1)
  (pk (noop))
  x)

(lambda* (#:optional xyz)
  (pk (noop))
  (define x 1))                ;; bad

(let ((x (lambda* (#:optional zzz) #f)))
  (x)
  (x 1))                       ;; notyet

(lambda* (#:optional lambda*-test5-arg) #f)
lambda*-test5-arg              ;; bad

(lambda* (#:optional x) lambda*-test6-var)
(define lambda*-test6-var #f)

(lambda* (#:optional x)
  (nosuchfunc))                ;; bad

(lambda* (a #:optional
	    (b nodef))         ;; bad
  #f)
(lambda* (#:key
	  (a nodef)            ;; bad
	  b)
  #f)

(lambda* (x y)   #f)           ;; bad
(lambda* (x . y) #f)           ;; bad

(lambda* (x #:optional (y x))
  (pk x)
  (pk y))


;;-----------------------------------------------------------------------------
;; let-keywords

(let ((rest '(1 2 3)))
  (let-keywords rest #t () #f)
  (let-keywords rest nodef () #f)                 ;; bad
  (let-keywords rest #t (x) #f)
  (let-keywords rest #t (x . y) #f)               ;; bad
  (let-keywords rest #t ((x 9) . y) #f)           ;; bad
  (let-keywords rest #t ((x 9) . (y '(9 9))) #f)  ;; bad

  (let-keywords rest #t (x x) #f)                 ;; bad
  (let-keywords rest #t ((x 99) x) #f)            ;; bad

  (let-keywords rest #t ((x x)) #f)               ;; bad
  (let-keywords rest #t ((x 99) (y x)) #f)        ;; bad
  )


;;-----------------------------------------------------------------------------
;; let-keywords*

(let ((rest '(#:x 1 #:y 2)))
  (let-keywords* rest #t () #f)
  (let-keywords* rest nodef () #f)                 ;; bad
  (let-keywords* rest #t (x) #f)
  (let-keywords* rest #t (x . y) #f)               ;; bad
  (let-keywords* rest #t ((x 9) . y) #f)           ;; bad
  (let-keywords* rest #t ((x 9) . (y '(9 9))) #f)  ;; bad

  (let-keywords* rest #t (x x) #f)
  (let-keywords* rest #t ((x 99) x) #f)

  (let-keywords* rest #t ((x x)) #f)               ;; bad
  (let-keywords* rest #t ((x 99) (y x)) #f)
  )


;;-----------------------------------------------------------------------------
;; let-optional

(let ((rest '(1 2 3)))
  (let-optional rest () #f)
  (let-optional rest (x) #f)
  (let-optional rest (x . y) #f)               ;; bad
  (let-optional rest ((x 9) . y) #f)           ;; bad
  (let-optional rest ((x 9) . (y '(9 9))) #f)  ;; bad

  (let-optional rest (x x) #f)                 ;; bad
  (let-optional rest ((x 99) x) #f)            ;; bad

  (let-optional rest ((x x)) #f)               ;; bad
  (let-optional rest ((x 99) (y x)) #f)        ;; bad
  )


;;-----------------------------------------------------------------------------
;; let-optional*

(let ((rest '(1 2 3)))
  (let-optional* rest () #f)
  (let-optional* rest (x) #f)
  (let-optional* rest (x . y) #f)               ;; bad
  (let-optional* rest ((x 9) . y) #f)           ;; bad
  (let-optional* rest ((x 9) . (y '(9 9))) #f)  ;; bad

  (let-optional* rest (x x) #f)
  (let-optional* rest ((x 99) x) #f)

  (let-optional* rest ((x x)) #f)               ;; bad
  (let-optional* rest ((x 99) (y x)) #f)
  )
