;; Checking (ice-9 threads).

;; Copyright 2003, 2004, 2005, 2006, 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint ice-9 threads)
  #:use-module (guile-lint common)
  #:use-module (ice-9 threads)
  #:use-module (srfi srfi-1))


;;-----------------------------------------------------------------------------
;; begin-thread

;; macro
(lint-arity! begin-thread '(1 0 #t))

(lint-types! begin-thread  type-any type-any)

(define (lint-begin-thread where now expr env)
  (lint-body-list where now (cdr expr) env))

(lint-handler! begin-thread lint-begin-thread)


;;-----------------------------------------------------------------------------
;; letpar

(define (lint-letpar where now expr env)
  (lint-let where now expr env))
      
(if (defined? 'letpar)  ;; new in guile 1.8
    (lint-handler! letpar lint-letpar))


;;-----------------------------------------------------------------------------
;; make-thread

;; macro
(lint-arity! make-thread '(1 0 #t))

(lint-types! make-thread   type-thread type-procedure type-any)

(define (lint-make-thread where now expr env)
  ;; preserve the `now' state, which would otherwise be lost across lambda
  ;; in the macro expansion
  (lint-expr where now (cdr expr) env))

(lint-handler! make-thread lint-make-thread)


;;-----------------------------------------------------------------------------
;; monitor

;; macro
(lint-arity! monitor '(1 0 #t))

(lint-types! monitor    type-any type-any)
(lint-handler! monitor  lint-begin-thread)


;;-----------------------------------------------------------------------------
;; n-for-each-par-map

(define (lint-n-for-each-par-map where now expr env)
  (lint-procexpr-arglist where now (third expr) env '(from-pproc))
  (lint-procexpr-arglist where now (fourth expr) env
			 (make-list (length (cddddr expr)) 'elem))
  (lint-list where now expr env))

(if (defined? 'n-for-each-par-map)  ;; new in guile 1.8
    (lint-handler! n-for-each-par-map lint-n-for-each-par-map))


;;-----------------------------------------------------------------------------
;; n-par-for-each

(define (lint-n-par-for-each where now expr env)
  (lint-procexpr-arglist where now (third expr) env
			 (make-list (length (cdddr expr)) 'elem))
  (lint-list where now expr env))

(if (defined? 'n-par-for-each)  ;; new in guile 1.8
    (lint-handler! n-par-for-each lint-n-par-for-each))


;;-----------------------------------------------------------------------------
;; n-par-map

(if (defined? 'n-par-map)  ;; new in guile 1.8
    (lint-handler! n-par-map lint-n-par-for-each))


;;-----------------------------------------------------------------------------
;; parallel

(define (lint-parallel where now expr env)
  (lint-list where now (cdr expr) env))
      
(if (defined? 'parallel)  ;; new in guile 1.8
    (begin
      (lint-arity! parallel '(1 0 #t))
      (lint-handler! parallel lint-parallel)))


;;-----------------------------------------------------------------------------
;; par-for-each

(if (defined? 'par-for-each)  ;; new in guile 1.8
    (lint-handler! par-for-each lint-for-each))


;;-----------------------------------------------------------------------------
;; par-map

(if (defined? 'par-map)  ;; new in guile 1.8
    (lint-handler! par-map lint-for-each))


;;-----------------------------------------------------------------------------
;; with-mutex

;; An empty body works due to `begin' now accepting that, but not sure if
;; this is meant to be a documented feature.  It doesn't really make sense
;; to acquire a mutex only to do nothing.

;; macro
(lint-arity! with-mutex '(2 0 #t))

(lint-types! with-mutex type-any type-mutex type-any)

(define (lint-with-mutex where now expr env)
  (lint-body-list where now (cdr expr) env
		  #:initial-used 1)) ;; the mutex arg

(lint-handler! with-mutex lint-with-mutex)
