;; Checking (ice-9 weak-vector).

;; Copyright 2007 Kevin Ryde

;; This file is part of Guile-Lint.
;;
;; Guile-Lint is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; Guile-Lint is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (guile-lint ice-9 weak-vector)
  #:use-module (guile-lint common))

(if (module-exists? '(ice-9 weak-vector)) ;; only in guile 1.8
    (use-modules (ice-9 weak-vector)))

;; the following new in guile 1.8
(if (defined? 'make-weak-key-alist-vector)
    (begin
      (define-public type-weak-key-alist-vector
	(make-type "weak key alist vector"
		   #:supertype type-vector
		   #:test weak-key-alist-vector?))

      (define-public type-weak-value-alist-vector
	(make-type "weak value alist vector"
		   #:supertype type-vector
		   #:test weak-value-alist-vector?))

      (define-public type-doubly-weak-alist-vector
	(make-type "weak value alist vector"
		   #:supertype type-vector
		   #:test doubly-weak-alist-vector?))

      (lint-types! make-weak-key-alist-vector
		   'pure type-weak-key-alist-vector
		   type-exact-integer-nonneg)
      (lint-types! make-weak-value-alist-vector
		   'pure type-weak-value-alist-vector
		   type-exact-integer-nonneg)
      (lint-types! make-doubly-weak-alist-vector
		   'pure type-doubly-weak-alist-vector
		   type-exact-integer-nonneg)

      (lint-types! weak-key-alist-vector?    'pure type-boolean type-any)
      (lint-types! weak-value-alist-vector?  'pure type-boolean type-any)
      (lint-types! doubly-weak-alist-vector? 'pure type-boolean type-any)))


;; the following in the core in guile 1.6

(define-public type-weak-vector
  (make-type "weak vector"
	     #:supertype type-vector
	     #:test weak-vector?))

(lint-types! make-weak-vector  'pure type-weak-vector
	     type-exact-integer-nonneg type-any)
(lint-types! list->weak-vector 'pure type-weak-vector type-list)
(lint-types! weak-vector   'pure type-weak-vector type-any)
(lint-types! weak-vector?  'pure type-boolean type-any)

